/*
 #
 #  File        : gmic.cpp
 #                ( C++ source file )
 #
 #  Description : GREYC's Magic Image Converter
 #                ( http://gmic.sourceforge.net )
 #                This file is a part of the CImg Library project.
 #                ( http://cimg.sourceforge.net )
 #
 #  Copyright   : David Tschumperle
 #                ( http://www.greyc.ensicaen.fr/~dtschump/ )
 #
 #  License     : CeCILL v2.0
 #                ( http://www.cecill.info/licences/Licence_CeCILL_V2-en.html )
 #
 #  This software is governed by the CeCILL  license under French law and
 #  abiding by the rules of distribution of free software.  You can  use,
 #  modify and/ or redistribute the software under the terms of the CeCILL
 #  license as circulated by CEA, CNRS and INRIA at the following URL
 #  "http://www.cecill.info".
 #
 #  As a counterpart to the access to the source code and  rights to copy,
 #  modify and redistribute granted by the license, users are provided only
 #  with a limited warranty  and the software's author,  the holder of the
 #  economic rights,  and the successive licensors  have only  limited
 #  liability.
 #
 #  In this respect, the user's attention is drawn to the risks associated
 #  with loading,  using,  modifying and/or developing or reproducing the
 #  software by the user in light of its specific status of free software,
 #  that may mean  that it is complicated to manipulate,  and  that  also
 #  therefore means  that it is reserved for developers  and  experienced
 #  professionals having in-depth computer knowledge. Users are therefore
 #  encouraged to load and test the software's suitability as regards their
 #  requirements in conditions enabling the security of their systems and/or
 #  data to be ensured and,  more generally, to use and operate it in the
 #  same conditions as regards security.
 #
 #  The fact that you are presently reading this means that you have had
 #  knowledge of the CeCILL license and that you accept its terms.
 #
*/

#define cimg_debug 1
#include "../CImg.h"
#include <math.h>
using namespace cimg_library;
// The undef below is necessary when using a non-standard compiler.
#ifdef cimg_use_visualcpp6
#define std
#endif
#ifdef min
#undef min
#undef max
#endif

// Global variables.
//------------------
#if defined(gmic_bool) || !defined(gmic_separate_compilation)
namespace gmic_globals {
  CImgList<unsigned int> primitives3d[4096];
  CImgList<unsigned char> colors3d[4096];
  CImg<float> points3d[4096];
  CImgList<char> filenames, command_line;
  unsigned int position = 1;
  bool released = false;
  char **argv = 0;
  int argc = 0;
}
#else
namespace gmic_globals {
  extern CImgList<unsigned int> primitives3d[4096];
  extern CImgList<unsigned char> colors3d[4096];
  extern CImg<float> points3d[4096];
  extern CImgList<char> filenames, command_line;
  extern unsigned int position;
  extern bool released;
  extern char **argv;
  extern int argc;
}
#endif
using namespace gmic_globals;

// Error procedure.
//-----------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
void gmic_error(const char *format, ...) {
  va_list ap;
  va_start(ap,format);
  std::fprintf(stderr,"\n<gmic-#%u> ** Error ** ",filenames.size);
  std::vfprintf(stderr,format,ap);
  va_end(ap);
  std::fprintf(stderr,"\n<gmic-#%u> Abort.\n",filenames.size);
  std::fflush(stderr);
  std::exit(0);
}
#else
void gmic_error(const char *format, ...);
#endif

// Warning procedure.
//-------------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
void gmic_warning(const char *format, ...) {
  static const bool quiet = cimg_option("-quiet",false,0);
  if (!quiet) {
    va_list ap;
    va_start(ap,format);
    std::fprintf(stderr,"\n<gmic-#%u> ** Warning ** ",filenames.size);
    std::vfprintf(stderr,format,ap);
    va_end(ap);
    std::fflush(stderr);
  }
}
#else
void gmic_warning(const char *format, ...);
#endif

// Print status messages.
//-----------------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
void gmic_print(const char *format, ...) {
  static const bool quiet = cimg_option("-quiet",false,0);
  if (!quiet) {
    va_list ap;
    va_start(ap,format);
    std::fprintf(stderr,"\n<gmic-#%u> ",filenames.size);
    std::vfprintf(stderr,format,ap);
    va_end(ap);
    std::fflush(stderr);
  }
}
#else
void gmic_print(const char *format, ...);
#endif

// Generic code to check validity of indices.
//-------------------------------------------
#define gmic_check_indice(ind,funcname) {	\
  if (ind<0) ind+=images.size; \
  if (ind<0 || ind>=(int)images.size) \
    gmic_error(funcname " : Invalid indice '[%d]' (maximum valid indice is %u).", \
               gmic_inds,ind,images.size-1); }

// Return indices of the images from a string.
//--------------------------------------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
CImg<unsigned int> gmic_indices(const char *const string, const unsigned int indice_max,
                                const char *const command) {
  if (!cimg::strlen(string)) return CImg<unsigned int>();
  CImgList<unsigned int> inds;
  const char *it = string;
  for (bool stopflag = false; !stopflag; ) {
    char sep = 0, end = 0, item0[256] = { 0 }, item1[256] = { 0 };
    float ind0 = 0, ind1 = 0, step = 1;
    if (std::sscanf(it,"%255[^,]%c",item0,&end)!=2) stopflag = true;
    else it+=1+cimg::strlen(item0);
    const int err = std::sscanf(item0,"%255[^:]%c%f%c",item1,&sep,&step,&end);
    if (err!=1 && err!=3) gmic_error("Command '%s' : Invalid indice(s) '[%s]'.",command,string);
    if (std::sscanf(item1,"%f%%-%f%c%c",&ind0,&ind1,&sep,&end)==3 && sep=='%') {
      ind0 = ind0*indice_max/101;
      ind1 = ind1*indice_max/101;
    } else if (std::sscanf(item1,"%f%%-%f%c",&ind0,&ind1,&end)==2)
      ind0 = ind0*indice_max/101;
    else if (std::sscanf(item1,"%f-%f%c%c",&ind0,&ind1,&sep,&end)==3 && sep=='%')
      ind1 = ind1*indice_max/101;
    else if (std::sscanf(item1,"%f-%f%c",&ind0,&ind1,&end)==2) { }
    else if (std::sscanf(item1,"%f%c%c",&ind0,&sep,&end)==2 && sep=='%')
      ind1 = (ind0 = ind0*indice_max/101);
    else if (std::sscanf(item1,"%f%c",&ind0,&end)==1)
      ind1 = ind0;
    else gmic_error("Command '%s' : Invalid indice(s) '[%s]'.",command,string);
    if (ind0<0) ind0+=indice_max;
    if (ind1<0) ind1+=indice_max;
    if (ind0<0 || ind0>=(int)indice_max || ind1<0 || ind1>=(int)indice_max || step<=0)
      if (indice_max) gmic_error("Command '%s' : Invalid indice(s) '[%s]' (maximum valid indice is %u).",
                                 command,string,indice_max-1);
      else gmic_error("Command '%s' : Invalid indice(s) '[%s]' (image list is empty).",
                      command,string);
    if (ind0==ind1) inds.insert(CImg<unsigned int>::vector((unsigned int)ind0));
    else {
      if (ind0>ind1) cimg::swap(ind0,ind1);
      inds.insert(CImg<unsigned int>::sequence((unsigned int)((step+ind1-ind0)/step),
					       (unsigned int)ind0,
					       (unsigned int)ind1).get_split('y'));
    }
  }
  inds = inds.get_append('y').sort().get_split('y');
  cimglist_for(inds,l) if (l!=inds.size-1 && inds(l,0)==inds(l+1,0)) inds.remove(l);
  return inds.get_append('y').sort();
}
#else
CImg<unsigned int> gmic_indices(const char *const string, const unsigned int indice_max,
                                const char *const command);
#endif

// Return stringified version of indices or filenames.
//----------------------------------------------------
#define gmic_inds gmic_string(indices,true)
#if defined(gmic_main) || !defined(gmic_separate_compilation)
char* gmic_string(const CImg<unsigned int>& indices, const bool display_indices) {
  static char res0[256] = { 0 }, res1[1024] = { 0 };
  const unsigned int siz = indices.size();
  if (display_indices) {
    switch (siz) {
    case 0: std::sprintf(res0," []"); break;
    case 1: std::sprintf(res0," [%u]",indices[0]); break;
    case 2: std::sprintf(res0,"s [%u,%u]",indices[0],indices[1]); break;
    case 3: std::sprintf(res0,"s [%u,%u,%u]",indices[0],indices[1],indices[2]); break;
    case 4: std::sprintf(res0,"s [%u,%u,%u,%u]",indices[0],indices[1],indices[2],indices[3]); break;
    default: std::sprintf(res0,"s [%u,...,%u]",indices[0],indices[siz-1]);
    }
    return res0;
  }
  switch (siz) {
  case 0: std::sprintf(res1," "); break;
  case 1: std::sprintf(res1,"%s",filenames[indices[0]].ptr()); break;
  case 2: std::sprintf(res1,"%s, %s",filenames[indices[0]].ptr(),filenames[indices[1]].ptr()); break;
  case 3: std::sprintf(res1,"%s, %s, %s",filenames[indices[0]].ptr(),filenames[indices[1]].ptr(),
                       filenames[indices[2]].ptr()); break;
  case 4: std::sprintf(res1,"%s, %s, %s, %s",filenames[indices[0]].ptr(),filenames[indices[1]].ptr(),
                       filenames[indices[2]].ptr(), filenames[indices[3]].ptr()); break;
  default: std::sprintf(res1,"%s, ..., %s",filenames[indices[0]].ptr(),filenames[indices[siz-1]].ptr());
  }
  return res1;
}
#else
char* gmic_string(const CImg<unsigned int>& indices, const bool display_indices);
#endif

// Generic code for simple commands that has no parameters and act on images.
//---------------------------------------------------------------------------
#define gmic_simple_item(option,function,description) \
  if ((*item0=='-' && !cimg::strcmp(#function,item0+1)) || !cimg::strcmp(option,item0)) { \
    gmic_print(description,gmic_inds); cimg_foroff(indices,l) images[indices[l]].function(); \
    continue; }

// Generic code used for the type cast command.
//---------------------------------------------
#define gmic_cast(pixel_type,st_type) \
  if (!cimg::strcmp(#pixel_type,argument)) { \
    gmic_print("Set pixel type to '%s'.",#pixel_type); ++position; \
    if (!cimg::strcmp(st_type,cimg::type<T>::string())) continue; \
    CImgList<pixel_type> casted_images; \
    while (images) { casted_images.insert(images[0]); images.remove(0); } \
    return gmic_parse_##pixel_type(casted_images); }

// Generic code used for arithmetic operators.
//--------------------------------------------
#define gmic_arithmetic_item(option1,option2,\
                             function1,description1,arg1_1,arg1_2,value_type1, \
                             function2,description2_1,description2_2,arg2_1,arg2_2,description3) \
 if (!cimg::strcmp(option1,item0) || !cimg::strcmp(option2,item0)) { \
   double value = 0; char inds[256] = { 0 }, sep = 0, end = 0; \
    if (std::sscanf(argument,"%lf%c",&value,&end)==1) { \
      gmic_print(description1 ".",arg1_1,arg1_2); \
      cimg_foroff(indices,l) images[indices[l]].function1((value_type1)value); \
      ++position; \
    } else if (std::sscanf(argument,"[%255[0-9.eE%+-]%c%c",inds,&sep,&end)==2 && sep==']') { \
      const CImg<unsigned int> ind = gmic_indices(inds,images.size,option1); \
      if (ind.size()!=1) gmic_error(description2_1 " : Argument '[%s]' should contain one indice.",gmic_inds,inds); \
      gmic_print(description2_2 ".",arg2_1,arg2_2); \
      const CImg<T> img0 = images[ind[0]]; \
      cimg_foroff(indices,l) images[indices[l]].function2(img0); \
      ++position; \
    } else { \
      gmic_print(description3 ".",gmic_inds); \
      if (images && indices) { \
        for (unsigned int siz = indices.size(), ind0 = indices[0], off = 0, l = 1; l<siz; ++l) { \
          const unsigned int ind = indices[l] - off; \
          images[ind0].function2(images[ind]); \
          images.remove(ind); filenames.remove(ind); \
          ++off; \
        }}} continue; }

// Display the image(s).
//----------------------
template<typename T>
bool gmic_display(const CImgList<T>& images, const CImg<unsigned int>& indices) {
  static const bool quiet = cimg_option("-quiet",false,0);
  if (!images || !indices) { gmic_print("Display image []."); return false; }
  CImgList<unsigned int> inds = indices.get_unroll('x').get_split('x');
  CImgList<T> visu;
  cimglist_for(inds,l) {
    const unsigned int ind = inds(l,0);
    const CImg<T> &img = images[ind];
    if (img) visu.insert(img,~0U,true);
    else { gmic_warning("Display image[%d] : Empty image, will be ignored.",ind); inds.remove(l--); }
  }
  const CImg<unsigned int> nindices = inds.get_append('x');
  const char *const fnames = gmic_string(nindices,false);
  gmic_print("Display image%s = '%s'.\n\n",gmic_inds,fnames);
  if (visu.size!=1) visu.display(fnames,1,!quiet,'x','p');
  else {
    const CImg<T> &img = visu[0];
    char title[1024] = { 0 };
    std::sprintf(title,"%s (%dx%dx%dx%d)",fnames,img.dimx(),img.dimy(),img.dimz(),img.dimv());
    img.display(title,1,!quiet);
  }
  return true;
}

// Display the 3D object(s).
//--------------------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
bool gmic_display3d(const CImg<unsigned int>& indices) {
  if (!indices) { gmic_print("Display 3D object []."); return false; }
  bool exist3d = false;
  cimg_foroff(indices,l) if (points3d[indices[l]].size()) { exist3d = true; break; }
  if (!exist3d) return false;
  static const char *background3d = cimg_option("-back3d","120,120,140",0);
  static const int
    render3d = cimg_option("-r3d",4,0),
    renderm3d = cimg_option("-rd3d",-1,0);
  static const bool orient3d = cimg_option("-orient3d",false,0);
  static const float
    specl3d = cimg_option("-specl3d",0.15f,0),
    specs3d = cimg_option("-specs3d",0.8f,0);
  CImg<unsigned char> background(cimg_fitscreen(640,480,1),1,3);
  int bgcolors[3] = { 0 }; std::sscanf(background3d,"%d%*c%d%*c%d",bgcolors,bgcolors+1,bgcolors+2);
  cimg_forV(background,k) background.get_shared_channel(k).fill(bgcolors[k]);
  CImgDisplay disp(background);
  cimg_foroff(indices,l) {
    const unsigned int ind = indices[l];
    if (points3d[ind]) {
      gmic_print("Display 3D object [%u] = '%s' (%d points, %u primitives).",
                 ind,filenames[ind].ptr(),points3d[ind].dimx(),primitives3d[ind].size);
      disp.set_title("%s (%d points, %u primitives)",
                     filenames[ind].ptr(),points3d[ind].dimx(),primitives3d[ind].size);
      background.display_object3d(disp,points3d[ind],primitives3d[ind],colors3d[ind],
                                  true,render3d,renderm3d,!orient3d,500,specl3d,specs3d);
    }
    if (disp.is_closed) break;
  }
  return true;
}
#else
bool gmic_display3d(const CImg<unsigned int>& indices);
#endif

// Plot the image(s).
//-------------------
template<typename T>
bool gmic_display_plot(const CImgList<T>& images, const CImg<unsigned int>& indices,
                       const unsigned int plot_type, const double vmin, const double vmax) {
  static const bool quiet = cimg_option("-quiet",false,0);
  if (!images || !indices) { gmic_print("Plot image []."); return false; }
  const unsigned char white[3] = { 255,255,255 };
  CImg<unsigned char> background(cimg_fitscreen(640,480,1),1,3,0);
  CImgDisplay disp(background);
  cimg_foroff(indices,l) {
    const unsigned int ind = indices[l];
    const CImg<T>& img = images[ind];
    gmic_print("Plot image%s = '%s'.\n",gmic_inds,gmic_string(indices,false));
    if (!quiet) { std::fputc('\n',stderr); img.print(filenames[ind].ptr()); }
    CImg<unsigned char> palette = CImg<unsigned char>(3,img.dim,1,1,180).noise(70,1);
    unsigned long trame = 0x0F0F0F0F;
    char message[1024];
    if (img.dimv()==1) { palette[0] = palette[1] = 140; palette[2] = 255; }
    else {
      palette(0,0) = 255; palette(1,0) = 30; palette(2,0) = 30;
      if (img.dimv()>1) { palette(0,1) = 30; palette(1,1) = 255; palette(2,1) = 30; }
      if (img.dimv()>2) { palette(0,2) = 30; palette(1,2) = 30; palette(2,2) = 255; }
    }
    char title[1024] = { 0 };
    std::sprintf(title,"%s (%dx%dx%dx%d)",
                 filenames[ind].ptr(),img.dimx(),img.dimy(),img.dimz(),img.dimv());
    disp.set_title(title);
    CImg<unsigned char> visu0, text;
    do {
      if (!visu0.ptr() || disp.is_resized) {
        disp.resize(false);
        visu0 = CImg<unsigned char>(disp.dimx(),disp.dimy(),1,3,0);
        cimg_forV(img,k) visu0.draw_graph(img.get_shared_channel(k),&palette(0,k),1,plot_type,vmax,vmin);
        if (vmin==vmax && vmin==0)
          visu0.draw_axis(CImg<unsigned int>::sequence(1+visu0.dimx()/60,0,(int)img.size()-1),
                          CImg<double>::sequence(1+visu0.dimy()/60,img.max(),img.min()),white);
        else
          visu0.draw_axis(CImg<unsigned int>::sequence(1+visu0.dimx()/60,0,(int)img.size()-1),
                          CImg<double>::sequence(1+visu0.dimy()/60,vmax,vmin),white);
      }
      CImg<unsigned char> visu(visu0);
      if (disp.mouse_x>=0) {
        visu.draw_line(disp.mouse_x,0,disp.mouse_x,visu.dimy()-1,white,0.7f,trame = cimg::rol(trame));
        const unsigned x = disp.mouse_x*img.size()/(img.dimv()*disp.dimx());
        std::sprintf(message,"Value[%d] = [ ",x);
        cimg_forV(img,k) std::sprintf(message+std::strlen(message),"%g ",(double)img(x,0,0,k));
        std::sprintf(message+std::strlen(message),"]");
        text.assign().draw_text(0,0,message,white,0,1);
        visu.draw_image((visu.dimx()-text.dimx())/2,0,text);
      } else visu = visu0;
      disp.display(visu).wait(40);
    } while (!disp.is_closed && !disp.key);
  }
  return true;
}

// Main parsing procedure.
//------------------------
void gmic_parse_bool(CImgList<bool> &images);
void gmic_parse_uchar(CImgList<unsigned char> &images);
void gmic_parse_char(CImgList<char> &images);
void gmic_parse_ushort(CImgList<unsigned short> &images);
void gmic_parse_short(CImgList<short> &images);
void gmic_parse_uint(CImgList<unsigned int> &images);
void gmic_parse_int(CImgList<int> &images);
void gmic_parse_float(CImgList<float> &images);
void gmic_parse_double(CImgList<double> &images);

template<typename T>
void gmic_parse(CImgList<T> &images) {
  const int no_ind = (int)(~0U>>1);
  static const bool
    quiet     = cimg_option("-quiet",false,0),
    fullnames = cimg_option("-fullnames",false,0);

  while ((int)position<argc) {

    // Read current item/command from the command line.
    const char
      *item = argv[position],
      *argument = argv[(int)position+1<argc?position+1:position];
    char item0[1024] = { 0 }, item1[256] = { 0 };
    CImg<unsigned int> indices;
    if (item[0]=='-' && item[1] && item[1]!='.') {
      char sep0 = 0, sep1 = 0, end = 0;
      const int err = std::sscanf(item,"%1023[^[]%c%255[0-9.eE%,:+-]%c%c",item0,&sep0,item1,&sep1,&end);
      if (err==1) indices = CImg<unsigned int>::sequence(images.size,0,images.size-1);
      else if (err==4 && sep1==']')
        indices = gmic_indices(item1,(!strcmp(item0,"-i")||!strcmp(item0,"-input"))?images.size+1:images.size,
                               item0);
      else { std::strcpy(item0,item); item1[0] = 0; }
    }
    ++position;

    // Special items to ignore (global options).
    if (!cimg::strcmp("-quiet",item) ||
        !cimg::strcmp("-orient3d",item) ||
        !cimg::strcmp("-fullnames",item) ||
        !cimg::strcmp("-h",item) ||
        !cimg::strcmp("-help",item) ||
        !cimg::strcmp("--help",item)) continue;
    if (!cimg::strcmp("-m",item) ||
        !cimg::strcmp("-macros",item) ||
        !cimg::strcmp("-back3d",item) ||
        !cimg::strcmp("-r3d",item) ||
        !cimg::strcmp("-rd3d",item) ||
        !cimg::strcmp("-specl3d",item) ||
        !cimg::strcmp("-specs3d",item)) { ++position; continue; }

    //----------------------
    // Arithmetic operators
    //----------------------

    // Common arithmetic operators.
    gmic_arithmetic_item("-add","-+",
                         operator+=,"Add %g to image%s",value,gmic_inds,T,
                         operator+=,"Add to image%s",
			 "Add image [%d] to image%s",ind[0],gmic_inds,
                         "Add image%s together");

    gmic_arithmetic_item("-sub","--",
                         operator-=,"Substract %g to image%s",value,gmic_inds,T,
                         operator-=,"Substract to image%s",
			 "Substract image [%d] to image%s",ind[0],gmic_inds,
                         "Substract image%s together");

    gmic_arithmetic_item("-mul","-*",
                         operator*=,"Multiply image%s by %g",gmic_inds,value,double,
                         mul,"Multiply image%s",
			 "Multiply image%s by image [%d]",gmic_inds,ind[0],
                         "Multiply image%s together");

    gmic_arithmetic_item("-div","-/",
                         operator/=,"Divide image%s by %g",gmic_inds,value,double,
                         div,"Divide image%s",
			 "Divide image%s by image [%d]",gmic_inds,ind[0],
                         "Divide image%s together");

    gmic_arithmetic_item("-pow","-^",
                         pow,"Compute image%s to the power of %g",gmic_inds,value,double,
                         pow,"Compute power of image%s",
			 "Compute image%s to the power of image [%d]",gmic_inds,ind[0],
                         "Compute the power of image%s together");

    gmic_arithmetic_item("-min","-min",
                         min,"Compute pointwise minimum between image%s and %g",gmic_inds,value,T,
                         min,"Compute pointwise minimum with image%s",
			 "Compute pointwise minimum between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute pointwise minimum between image%s together");

    gmic_arithmetic_item("-max","-max",
                         max,"Compute pointwise maximum between image%s and %g",gmic_inds,value,T,
                         max,"Compute pointwise maximum with image%s",
			 "Compute pointwise maximum between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute pointwise maximum between image%s together");

    gmic_arithmetic_item("-mod","-%",
                         operator%=,"Compute pointwise modulo between image%s and %g.",gmic_inds,value,T,
                         operator%=,"Compute pointwise modulo with image%s",
			 "Compute pointwise modulo between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute pointwise modulo between image%s together");

    gmic_arithmetic_item("-and","-and",
                         operator&=,"Compute bitwise AND between image%s and %g.",gmic_inds,value,unsigned int,
                         operator&=,"Compute bitwise AND with image%s",
			 "Compute bitwise AND between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute bitwise AND between image%s together.");

    gmic_arithmetic_item("-or","-or",
                         operator|=,"Compute bitwise OR between image%s and %g.",gmic_inds,value,unsigned int,
                         operator|=,"Compute bitwise OR with image%s",
			 "Compute bitwise OR between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute bitwise OR between image%s together.");

    gmic_arithmetic_item("-xor","-xor",
                         operator^=,"Compute bitwise XOR between image%s and %g.",gmic_inds,value,unsigned int,
                         operator^=,"Compute bitwise XOR with image%s",
			 "Compute bitwise XOR between image%s and image [%d]",gmic_inds,ind[0],
                         "Compute bitwise XOR between image%s together.");

    // Other arithmetic operators.
    gmic_simple_item("-cos",cos,"Compute cosine of image%s.");
    gmic_simple_item("-sin",sin,"Compute sine of image%s.");
    gmic_simple_item("-tan",tan,"Compute tangent of image%s.");
    gmic_simple_item("-acos",acos,"Compute arccosine of image%s.");
    gmic_simple_item("-asin",asin,"Compute arcsine of image%s.");
    gmic_simple_item("-atan",atan,"Compute arctangent of image%s.");
    gmic_simple_item("-abs",abs,"Compute absolute value of image%s.");
    gmic_simple_item("-sqr",sqr,"Compute square function of image%s.");
    gmic_simple_item("-sqrt",sqrt,"Compute square root of image%s.");
    gmic_simple_item("-exp",exp,"Compute exponential of image%s.");
    gmic_simple_item("-log",log,"Compute logarithm of image%s.");
    gmic_simple_item("-log10",log10,"Compute logarithm_10 of image%s.");

    //---------------------------------------
    // Pointwise operations on pixel values
    //---------------------------------------

    // Type cast.
    if (!cimg::strcmp("-type",item) || !cimg::strcmp("-t",item)) {
      typedef unsigned char uchar;
      typedef unsigned short ushort;
      typedef unsigned int uint;
#ifndef gmic_debug
      gmic_cast(bool,"bool");
      gmic_cast(uchar,"unsigned char");
      gmic_cast(char,"char");
      gmic_cast(ushort,"unsigned short");
      gmic_cast(short,"short");
      gmic_cast(uint,"unsigned int");
      gmic_cast(int,"int");
      gmic_cast(double,"double");
#endif
      gmic_cast(float,"float");
      gmic_error("Set pixel type : Invalid argument '%s' "
                 "(should be '{bool,uchar,char,ushort,short,uint,int,float,double}').",
                 argument);
    }

    // Invert endianness.
    gmic_simple_item("-endian",invert_endianness,"Invert endianness of image%s.");

    // Fill.
    if (!cimg::strcmp("-fill",item0)) {
      char sep = 0, end = 0; int ind0 = 0;
      if (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']') {
	gmic_check_indice(ind0,"Fill image%s");
        gmic_print("Fill image%s with values of image [%d].",gmic_inds,ind0);
        const CImg<T> values = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].fill(values,true);
      } else {
        gmic_print("Fill image%s with values '%s'.",gmic_inds,argument);
        cimg_foroff(indices,l) images[indices[l]].fill(argument,true);
      }
      ++position; continue;
    }

    // Threshold.
    if (!cimg::strcmp("-threshold",item0)) {
      char sep = 0, end = 0; double value = 0;
      if ((std::sscanf(argument,"%lf%c%c",&value,&sep,&end)==2 && sep=='%') ||
          std::sscanf(argument,"%lf%c",&value,&end)==1) {
        gmic_print("Threshold image%s with %g%s.",gmic_inds,value,sep=='%'?"%":"");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          double vmin = 0, vmax = 0, nvalue = value;
          if (sep=='%') { vmin = img.minmax(vmax); nvalue = vmin + (vmax - vmin)*value/100; }
          img.threshold((T)nvalue);
        }
        ++position;
      } else {
        gmic_print("Threshold image%s : Interactive mode.",gmic_inds);
        CImgDisplay disp;
        char title[1024] = { 0 };
        cimg_foroff(indices,l) {
          CImg<T>
            &img = images[indices[l]],
            visu = img.depth>1?img.get_projections2d(img.dimx()/2,img.dimy()/2,img.dimz()/2).
            channels(0,cimg::min(3,img.dimv())-1):img.get_channels(0,cimg::min(3,img.dimv()-1));
          if (disp) disp.resize(cimg_fitscreen(visu.dimx(),visu.dimy(),1),false);
          else disp.assign(cimg_fitscreen(visu.dimx(),visu.dimy(),1),0,1);
          double
            vmin = 0, vmax = (double)img.maxmin(vmin),
            distmax = std::sqrt(cimg::sqr(disp.dimx()-1.0) + cimg::sqr(disp.dimy()-1.0)),
            amount = 50;
          bool stopflag = false, obutt = false;
          int omx = -1, omy = -1;
          CImg<T> res;
          for (disp.show().button = disp.key = 0; !stopflag; ) {
            const unsigned int key = disp.key;
            if (!res) {
              std::sprintf(title,"%s : Interactive threshold %.3g%%",filenames[indices[l]].ptr(),amount);
              disp.display(res=visu.get_threshold((T)(vmin + amount*(vmax-vmin)/100))).
                set_title(title).wait();
            }
            const int mx = disp.mouse_x, my = disp.mouse_y;
            if (disp.button && mx>=0 && my>=0) {
              if (omx==mx && omy==my && !obutt) break;
              omx = mx; omy = my; obutt = true;
              const double dist = std::sqrt((double)cimg::sqr(mx) + cimg::sqr(my));
              amount = dist*100/distmax;
              res.assign();
            } else if (!disp.button) obutt = false;
            if (disp.is_closed || (key && key!=cimg::keyCTRLLEFT)) stopflag = true;
            if (key==cimg::keyD && disp.is_keyCTRLLEFT &&
                (disp.resize(cimg_fitscreen(3*disp.width/2,3*disp.height/2,1),stopflag=false).key=0)==0)
              disp.is_resized = true;
            if (key==cimg::keyC && disp.is_keyCTRLLEFT &&
                (disp.resize(cimg_fitscreen(2*disp.width/3,2*disp.height/3,1),stopflag=false).key=0)==0)
              disp.is_resized = true;
            if (disp.is_resized) {
              disp.resize(false).display(res);
              distmax = std::sqrt(cimg::sqr(disp.dimx()-1.0) + cimg::sqr(disp.dimy()-1.0));
            }
          }
          img.threshold((T)(vmin + amount*(vmax-vmin)/100));
        }
      }
      continue;
    }

    // Cut.
    if (!cimg::strcmp("-cut",item0)) {
      char sep0 = 0, sep1 = 0, end = 0, arg0[256] = { 0 }, arg1[256] = { 0 };
      double value0 = 0, value1 = 0; int ind0 = no_ind, ind1 = no_ind;
      if (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",arg0,arg1,&end)==2 &&
          ((std::sscanf(arg0,"[%d%c%c",&ind0,&sep0,&end)==2 && sep0==']') ||
           (std::sscanf(arg0,"%lf%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
           std::sscanf(arg0,"%lf%c",&value0,&end)==1) &&
          ((std::sscanf(arg1,"[%d%c%c",&ind1,&sep1,&end)==2 && sep1==']') ||
           (std::sscanf(arg1,"%lf%c%c",&value1,&sep1,&end)==2 && sep1=='%') ||
           std::sscanf(arg1,"%lf%c",&value1,&end)==1)) {
        if (ind0!=no_ind) { gmic_check_indice(ind0,"Cut image%s"); value0 = images[ind0].min(); sep0 = 0; }
        if (ind1!=no_ind) { gmic_check_indice(ind1,"Cut image%s"); value1 = images[ind1].max(); sep1 = 0; }
        gmic_print("Cut image%s in [%g%s,%g%s].",gmic_inds,value0,sep0=='%'?"%":"",value1,sep1=='%'?"%":"");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          double vmin = 0, vmax = 0, nvalue0 = value0, nvalue1 = value1;
          if (sep0=='%') { vmin = img.minmax(vmax); nvalue0 = vmin + (vmax - vmin)*value0/100; }
          if (sep1=='%') { vmin = img.minmax(vmax); nvalue1 = vmin + (vmax - vmin)*value1/100; }
          img.cut((T)nvalue0,(T)nvalue1);
        }
        ++position;
      } else {
        gmic_print("Cut image%s : Interactive mode.",gmic_inds);
        CImgDisplay disp;
        char title[1024] = { 0 };
        cimg_foroff(indices,l) {
          CImg<T>
            &img = images[indices[l]],
            visu = img.depth>1?img.get_projections2d(img.dimx()/2,img.dimy()/2,img.dimz()/2).
            channels(0,cimg::min(3,img.dimv())-1):img.get_channels(0,cimg::min(3,img.dimv()-1));
          if (disp) disp.resize(cimg_fitscreen(visu.dimx(),visu.dimy(),1),false);
          else disp.assign(cimg_fitscreen(visu.dimx(),visu.dimy(),1),0,1);
          double vmin = 0, vmax = (double)img.maxmin(vmin), amount0 = 0, amount1 = 100;
          bool stopflag = false, obutt = false;
          int omx = -1, omy = -1;
          CImg<T> res;
          for (disp.show().button = disp.key = 0; !stopflag; ) {
            const unsigned int key = disp.key;
            if (!res) {
              std::sprintf(title,"%s : Interactive cut [%.3g%%,%.3g%%]",
                           filenames[indices[l]].ptr(),amount0,amount1);
              disp.display(res = visu.get_cut((T)(vmin + amount0*(vmax-vmin)/100),
                                              (T)(vmin + amount1*(vmax-vmin)/100))).
                set_title(title).wait();
            }
            const int mx = disp.mouse_x, my = disp.mouse_y;
            if (disp.button && mx>=0 && my>=0) {
              if (omx==mx && omy==my && !obutt) break;
              omx = mx; omy = my; obutt = true;
              amount0 = mx*100/disp.dimx(); amount1 = my*100/disp.dimy();
              res.assign();
            } else if (!disp.button) obutt = false;
            if (disp.is_closed || (key && key!=cimg::keyCTRLLEFT)) stopflag = true;
            if (key==cimg::keyD && disp.is_keyCTRLLEFT &&
                (disp.resize(cimg_fitscreen(3*disp.width/2,3*disp.height/2,1),stopflag=false).key=0)==0)
              disp.is_resized = true;
            if (key==cimg::keyC && disp.is_keyCTRLLEFT &&
                (disp.resize(cimg_fitscreen(2*disp.width/3,2*disp.height/3,1),stopflag=false).key=0)==0)
              disp.is_resized = true;
            if (disp.is_resized) disp.resize(false).display(res);
          }
          img.cut((T)(vmin + amount0*(vmax-vmin)/100),(T)(vmin + amount1*(vmax-vmin)/100));
        }
      }
      continue;
    }

    // Normalize.
    if (!cimg::strcmp("-normalize",item0) || !cimg::strcmp("-n",item0)) {
      char sep0 = 0, sep1 = 0, end = 0, arg0[256] = { 0 }, arg1[256] = { 0 };
      double value0 = 0, value1 = 0; int ind0 = no_ind, ind1 = no_ind;
      if (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",arg0,arg1,&end)==2 &&
          ((std::sscanf(arg0,"[%d%c%c",&ind0,&sep0,&end)==2 && sep0==']') ||
           (std::sscanf(arg0,"%lf%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
           std::sscanf(arg0,"%lf%c",&value0,&end)==1) &&
          ((std::sscanf(arg1,"[%d%c%c",&ind1,&sep1,&end)==2 && sep1==']') ||
           (std::sscanf(arg1,"%lf%c%c",&value1,&sep1,&end)==2 && sep1=='%') ||
           std::sscanf(arg1,"%lf%c",&value1,&end)==1)) {
        if (ind0!=no_ind) { gmic_check_indice(ind0,"Normalize image%s"); value0 = images[ind0].min(); sep0 = 0; }
        if (ind1!=no_ind) { gmic_check_indice(ind1,"Normalize image%s"); value1 = images[ind1].max(); sep1 = 0; }
        gmic_print("Normalize image%s in [%g%s,%g%s].",gmic_inds,value0,sep0=='%'?"%":"",value1,sep1=='%'?"%":"");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          double vmin = 0, vmax = 0, nvalue0 = value0, nvalue1 = value1;
          if (sep0=='%') { vmin = img.minmax(vmax); nvalue0 = vmin + (vmax - vmin)*value0/100; }
          if (sep1=='%') { vmin = img.minmax(vmax); nvalue1 = vmin + (vmax - vmin)*value1/100; }
          img.normalize((T)nvalue0,(T)nvalue1);
        }
      } else gmic_error("Normalize image%s : Invalid argument '%s' "
                        "(should be 'value0[%],value1[%]').",gmic_inds,argument);
      ++position; continue;
    }

    // Equalize histogram.
    if (!cimg::strcmp("-equalize",item0)) {
      int nb = 256;
      if (std::sscanf(argument,"%d",&nb)>0) ++position;
      if (nb<=0) gmic_error("Equalize image%s with %d clusters : Invalid number of clusters.",gmic_inds,nb);
      gmic_print("Equalize image%s with %d clusters.",gmic_inds,nb);
      cimg_foroff(indices,l) images[indices[l]].equalize(nb);
      continue;
    }

    // Quantize.
    if (!cimg::strcmp("-quantize",item0)) {
      int nb = 0; char end = 0;
      if (std::sscanf(argument,"%d%c",&nb,&end)==1) {
        if (nb<=0) gmic_error("Quantize image%s with %d levels : Invalid number of levels.",gmic_inds,nb);
        gmic_print("Quantize image%s with %d levels.",gmic_inds,nb);
        cimg_foroff(indices,l) images[indices[l]].quantize(nb);
      } else gmic_error("Quantize image%s : Invalid argument '%s' "
                        "(should be 'nb_levels').",gmic_inds,argument);
      ++position; continue;
    }

    // Add noise.
    if (!cimg::strcmp("-noise",item0)) {
      float sigma = 0; char sep = 0, end = 0; int ntype = 0;
      if ((std::sscanf(argument,"%f%c%*c%d%c",&sigma,&sep,&ntype,&end)==3 && sep=='%') ||
          std::sscanf(argument,"%f%*c%d%c",&sigma,&ntype,&end)==2 ||
          (std::sscanf(argument,"%f%c%c",&sigma,&sep,&end)==2 && sep=='%') ||
          std::sscanf(argument,"%f%c",&sigma,&end)==1) {
        const char *st_type = ntype==0?"gaussian":ntype==1?"uniform":ntype==2?"salt&pepper":"poisson";
        if (ntype<0 || ntype>3) gmic_error("Add noise with variance %g%s to image%s : Invalid noise type %d.",
                                           sigma,sigma<0?"%":"",gmic_inds,ntype);
        if (sigma<0) gmic_error("Add %s noise with variance %g%s to image%s : Invalid variance.",
                                st_type,sigma,sigma<0?"%":"",gmic_inds);
        if (sep=='%') sigma = -sigma;
        gmic_print("Add %s noise with variance %g%s to image%s.",
                   st_type,cimg::abs(sigma),sigma<0?"%":"",gmic_inds);
        cimg_foroff(indices,l) images[indices[l]].noise(sigma,ntype);
      } else gmic_error("Add noise to image%s : Invalid argument '%s' "
                        "(should be 'variance[%][,noise_type]').",gmic_inds,argument);
      ++position; continue;
    }

    // Compute pointwise norms and orientations.
    gmic_simple_item("-norm",pointwise_norm,"Compute vector norm.");
    gmic_simple_item("-orientation",pointwise_orientation,"Compute vector orientation.");

    //------------------------
    // Color base conversions
    //------------------------
    gmic_simple_item("-rgb2hsv",RGBtoHSV,"Convert image%s from RGB to HSV colorbases.");
    gmic_simple_item("-rgb2hsl",RGBtoHSL,"Convert image%s from RGB to HSL colorbases.");
    gmic_simple_item("-rgb2hsi",RGBtoHSI,"Convert image%s from RGB to HSI colorbases.");
    gmic_simple_item("-rgb2yuv",RGBtoYUV,"Convert image%s from RGB to YUV colorbases.");
    gmic_simple_item("-rgb2ycbcr",RGBtoYCbCr," Convert image%s from RGB to YCbCr colorbases.");
    gmic_simple_item("-rgb2xyz",RGBtoXYZ,"Convert image%s from RGB to XYZ colorbases.");
    gmic_simple_item("-rgb2lab",RGBtoLab,"Convert image%s from RGB to Lab colorbases.");
    gmic_simple_item("-rgb2cmy",RGBtoCMY,"Convert image%s from RGB to CMY colorbases.");
    gmic_simple_item("-rgb2cmyk",RGBtoCMYK,"Convert image%s from RGB to CMYK colorbases.");
    gmic_simple_item("-cmyk2rgb",CMYKtoRGB,"Convert image%s from CMYK to RGB colorbases.");
    gmic_simple_item("-cmy2rgb",CMYtoRGB,"Convert image%s from CMY to RGB colorbases.");
    gmic_simple_item("-lab2rgb",LabtoRGB,"Convert image%s from Lab to RGB colorbases.");
    gmic_simple_item("-xyz2rgb",XYZtoRGB,"Convert image%s from XYZ to RGB colorbases.");
    gmic_simple_item("-ycbcr2rgb",YCbCrtoRGB,"Convert image%s from YCbCr to RGB colorbases.");
    gmic_simple_item("-yuv2rgb",YUVtoRGB,"Convert image%s from YUV to RGB colorbases.");
    gmic_simple_item("-hsi2rgb",HSItoRGB,"Convert image%s from HSI to RGB colorbases.");
    gmic_simple_item("-hsl2rgb",HSLtoRGB,"Convert image%s from HSL to RGB colorbases.");
    gmic_simple_item("-hsv2rgb",HSVtoRGB,"Convert image%s from HSV to RGB colorbases.");

    // Apply LUT.
    if (!cimg::strcmp("-lut2rgb",item0)) {
      int nb = 0, ind0 = 0; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']') {
	gmic_check_indice(ind0,"Map LUT on image%s");
        gmic_print("Map LUT [%d] on image%s.",ind0,gmic_inds);
	const CImg<T> palette = images[ind0];
	cimg_foroff(indices,l) images[indices[l]].pointwise_norm().LUTtoRGB(palette);
      } else if (std::sscanf(argument,"%d%c",&nb,&end)==1) {
        if (nb<0 || nb>2) gmic_error("Map LUT on image%s : Invalid LUT number %d.",gmic_inds,nb);
        gmic_print("Map %s LUT on image%s.",nb==0?"default":nb==1?"rainbow":"cluster",gmic_inds);
        cimg_foroff(indices,l) images[indices[l]].pointwise_norm().
          LUTtoRGB(nb==0?CImg<T>::default_LUT8():nb==1?CImg<T>::rainbow_LUT8():CImg<T>::contrast_LUT8());
      } else gmic_error("Map LUT on image%s : Invalid argument '%s' "
                        "(should be '[indice]' or '{0,1,2}').",gmic_inds,argument);
      ++position; continue;
    }

    // Convert to LUT.
    if (!cimg::strcmp("-rgb2lut",item0)) {
      int nb = 0, ind0 = 0, dithering = 0; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d%c%*c%d",&ind0,&sep,&dithering)>=2 && sep==']') {
	gmic_check_indice(ind0,"Index image%s with LUT");
        gmic_print("Index image%s with LUT [%d] %s dithering.",gmic_inds,ind0,dithering?"with":"without");
	const CImg<T> palette = images[ind0];
	cimg_foroff(indices,l) images[indices[l]].RGBtoLUT(palette,dithering,true);
      } else if (std::sscanf(argument,"%d%*c%d%c",&nb,&dithering,&end)==2 ||
                 std::sscanf(argument,"%d%c",&nb,&end)==1) {
        if (nb<0 || nb>2) gmic_error("Index image%s with LUT : Invalid LUT number %d.",gmic_inds,nb);
        gmic_print("Index image%s with %s LUT.",gmic_inds,nb==0?"default":nb==1?"rainbow":"cluster");
        cimg_foroff(indices,l) images[indices[l]].
          RGBtoLUT(nb==0?CImg<T>::default_LUT8():nb==1?CImg<T>::rainbow_LUT8():CImg<T>::contrast_LUT8(),
                   dithering,true);
      } else gmic_error("Index image%s with LUT : Invalid argument '%s' "
                        "(should be '[indice][,dithering]', or '{0,1,2}[,dithering]').",gmic_inds,argument);
      ++position; continue;
    }

    //-----------------------
    // Geometric manipulation
    //-----------------------

    // Resize.
    if (!cimg::strcmp("-resize",item0) || !cimg::strcmp("-r",item0)) {
      char
	sepx = 0, sepy = 0, sepz = 0, sepv = 0, end = 0,
	argx[256] = { 0 }, argy[256] = { 0 }, argz[256] = { 0 }, argv[256] = { 0 };
      float valx = 0, valy = 0, valz = 0, valv = 0;
      int interpolation = 1, borders = -1, center = 0, indx = no_ind, indy = no_ind, indz = no_ind, indv = no_ind;
      if ((std::sscanf(argument,"[%d%c%*c%d%*c%d%*c%d%c",&indx,&sepx,&interpolation,&borders,&center,&end)==5 ||
	   std::sscanf(argument,"[%d%c%*c%d%*c%d%c",&indx,&sepx,&interpolation,&borders,&end)==4 ||
	   std::sscanf(argument,"[%d%c%*c%d%c",&indx,&sepx,&interpolation,&end)==3 ||
	   std::sscanf(argument,"[%d%c%c",&indx,&sepx,&end)==2) && sepx==']') {
	gmic_check_indice(indx,"Resize image%s");
	if (interpolation<-1 || interpolation>5)
          gmic_error("Resize image%s : Invalid interpolation type '%d' "
                     "(should be '0','1','2','3','4' or '5').",gmic_inds,interpolation);
	const int
          ivalx = images[indx].dimx(),
          ivaly = images[indx].dimy(),
          ivalz = images[indx].dimz(),
          ivalv = images[indx].dimv();
        gmic_print("Resize image%s to %dx%dx%dx%d with %s interpolation.",
                   gmic_inds,ivalx,ivaly,ivalz,ivalv,
                   interpolation==0?"no":interpolation==1?"nearest neighbor":
                   interpolation==2?"moving average":interpolation==3?"linear":
                   interpolation==4?"grid":"cubic");
	cimg_foroff(indices,l)
          images[indices[l]].resize(ivalx,ivaly,ivalz,ivalv,interpolation,borders,center?true:false);
        ++position;
      } else if (((std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%d%*c%d%*c%d%c",
			       argx,argy,argz,argv,&interpolation,&borders,&center,&end)==7 ||
		   std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%d%*c%d%c",
			       argx,argy,argz,argv,&interpolation,&borders,&end)==6 ||
		   std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%d%c",
			       argx,argy,argz,argv,&interpolation,&end)==5 ||
		   std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",
			       argx,argy,argz,argv,&end)==4) &&
		  ((std::sscanf(argx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']') ||
		   (std::sscanf(argx,"%f%c%c",&valx,&sepx,&end)==2 && sepx=='%') ||
		   std::sscanf(argx,"%f%c",&valx,&end)==1) &&
		  ((std::sscanf(argy,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']') ||
		   (std::sscanf(argy,"%f%c%c",&valy,&sepy,&end)==2 && sepy=='%') ||
		   std::sscanf(argy,"%f%c",&valy,&end)==1) &&
		  ((std::sscanf(argz,"[%d%c%c",&indz,&sepz,&end)==2 && sepz==']') ||
		   (std::sscanf(argz,"%f%c%c",&valz,&sepz,&end)==2 && sepz=='%') ||
		   std::sscanf(argz,"%f%c",&valz,&end)==1) &&
		  ((std::sscanf(argv,"[%d%c%c",&indv,&sepv,&end)==2 && sepv==']') ||
		   (std::sscanf(argv,"%f%c%c",&valv,&sepv,&end)==2 && sepv=='%') ||
		   std::sscanf(argv,"%f%c",&valv,&end)==1)) ||
		 ((std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",
			       argx,argy,argz,&end)==3) &&
		  ((std::sscanf(argx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']') ||
		   (std::sscanf(argx,"%f%c%c",&valx,&sepx,&end)==2 && sepx=='%') ||
		   std::sscanf(argx,"%f%c",&valx,&end)==1) &&
		  ((std::sscanf(argy,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']') ||
		   (std::sscanf(argy,"%f%c%c",&valy,&sepy,&end)==2 && sepy=='%') ||
		   std::sscanf(argy,"%f%c",&valy,&end)==1) &&
		  ((std::sscanf(argz,"[%d%c%c",&indz,&sepz,&end)==2 && sepz==']') ||
		   (std::sscanf(argz,"%f%c%c",&valz,&sepz,&end)==2 && sepz=='%') ||
		   std::sscanf(argz,"%f%c",&valz,&end)==1)) ||
		 ((std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",
			       argx,argy,&end)==2) &&
		  ((std::sscanf(argx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']') ||
		   (std::sscanf(argx,"%f%c%c",&valx,&sepx,&end)==2 && sepx=='%') ||
		   std::sscanf(argx,"%f%c",&valx,&end)==1) &&
		  ((std::sscanf(argy,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']') ||
		   (std::sscanf(argy,"%f%c%c",&valy,&sepy,&end)==2 && sepy=='%') ||
		   std::sscanf(argy,"%f%c",&valy,&end)==1)) ||
		 ((std::sscanf(argument,"%255[][0-9.eE%+-]%c",
			       argx,&end)==1) &&
		  ((std::sscanf(argx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']') ||
		   (std::sscanf(argx,"%f%c%c",&valx,&sepx,&end)==2 && sepx=='%') ||
		   std::sscanf(argx,"%f%c",&valx,&end)==1))) {
	if (indx!=no_ind) { gmic_check_indice(indx,"Resize image%s"); valx = (float)images[indx].dimx(); sepx = 0; }
	if (indy!=no_ind) { gmic_check_indice(indy,"Resize image%s"); valy = (float)images[indy].dimy(); sepy = 0; }
	if (indz!=no_ind) { gmic_check_indice(indz,"Resize image%s"); valz = (float)images[indz].dimz(); sepz = 0; }
	if (indv!=no_ind) { gmic_check_indice(indv,"Resize image%s"); valv = (float)images[indv].dimv(); sepv = 0; }
        if (valx<0 || valy<0 || valz<0 || valv<0)
          gmic_error("Resize image%s : Invalid size '%g%sx%g%sx%g%sx%g%s'.",gmic_inds,
                     valx,sepx=='%'?"%":"",valy,sepy=='%'?"%":"",valz,sepz=='%'?"%":"",valv,sepv=='%'?"%":"");
        if (interpolation<-1 || interpolation>5)
          gmic_error("Resize image%s : Invalid interpolation type '%d' "
                     "(should be '0','1','2','3','4' or '5').",gmic_inds,interpolation);
        if (!valx) { valx = 100; sepx = '%'; }
        if (!valy) { valy = 100; sepy = '%'; }
        if (!valz) { valz = 100; sepz = '%'; }
        if (!valv) { valv = 100; sepv = '%'; }
        gmic_print("Resize image%s to %g%s%g%s%g%s%g%s with %s interpolation.",
                   gmic_inds,valx,sepx=='%'?"%x":"x",valy,sepy=='%'?"%x":"x",valz,
                   sepz=='%'?"%x":"x",valv,sepv=='%'?"% ":" ",
                   interpolation==0?"no":interpolation==1?"nearest neighbor":
                   interpolation==2?"moving average":interpolation==3?"linear":
                   interpolation==4?"grid":"cubic");

        cimg_foroff(indices,l) {
          CImg<T>& img = images[indices[l]];
          const int
            ivalx0 = (int)round(sepx=='%'?valx*img.width/100:valx),
            ivaly0 = (int)round(sepy=='%'?valy*img.height/100:valy),
            ivalz0 = (int)round(sepz=='%'?valz*img.depth/100:valz),
            ivalv0 = (int)round(sepv=='%'?valv*img.dim/100:valv),
            ivalx = ivalx0?ivalx0:1,
            ivaly = ivaly0?ivaly0:1,
            ivalz = ivalz0?ivalz0:1,
            ivalv = ivalv0?ivalv0:1;
          img.resize(ivalx,ivaly,ivalz,ivalv,interpolation,borders,center?true:false);
        }
        ++position;
      } else {
        gmic_print("Resize image%s : Interactive mode.",gmic_inds);
        char title[1024] = { 0 };
        cimg_foroff(indices,l) {
          CImg<T>& img = images[indices[l]];
          CImgDisplay disp(img,0,1);
          std::sprintf(title,"%s : Interactive resize",filenames[indices[l]].ptr());
          disp.set_title(title);
          img.get_user_coordinates(0,disp);
          gmic_print("Resize image [%d] to %dx%d.",indices[l],disp.dimx(),disp.dimy());
          img.resize(disp);
        }
      }
      continue;
    }

    // Resize2x. and Resize3x.
    gmic_simple_item("-resize2x",resize_doubleXY,"Resize image%s using Scale2x algorithm.");
    gmic_simple_item("-resize3x",resize_doubleXY,"Resize image%s using Scale3x algorithm.");

    // Crop.
    if (!cimg::strcmp("-crop",item0) || !cimg::strcmp("-c",item0)) {
      char st0[256] = { 0 }, st1[256] = { 0 }, st2[256] = { 0 }, st3[256] = { 0 };
      char st4[256] = { 0 }, st5[256] = { 0 }, st6[256] = { 0 }, st7[256] = { 0 };
      char sep0 = 0, sep1 = 0, sep2 = 0, sep3 = 0, sep4 = 0, sep5 = 0, sep6 = 0, sep7 = 0, end = 0;
      float a0 = 0, a1 = 0, a2 = 0, a3 = 0, a4 = 0, a5 = 0, a6 = 0, a7 = 0; int borders = 0;
      if ((std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                       "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%c",
                       st0,st1,st2,st3,st4,st5,st6,st7,&borders,&end)==9 ||
           std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                       "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",
                       st0,st1,st2,st3,st4,st5,st6,st7,&end)==8) &&
          (std::sscanf(st0,"%f%c",&a0,&end)==1 || (std::sscanf(st0,"%f%c%c",&a0,&sep0,&end)==2 && sep0=='%')) &&
          (std::sscanf(st1,"%f%c",&a1,&end)==1 || (std::sscanf(st1,"%f%c%c",&a1,&sep1,&end)==2 && sep1=='%')) &&
          (std::sscanf(st2,"%f%c",&a2,&end)==1 || (std::sscanf(st2,"%f%c%c",&a2,&sep2,&end)==2 && sep2=='%')) &&
          (std::sscanf(st3,"%f%c",&a3,&end)==1 || (std::sscanf(st3,"%f%c%c",&a3,&sep3,&end)==2 && sep3=='%')) &&
          (std::sscanf(st4,"%f%c",&a4,&end)==1 || (std::sscanf(st4,"%f%c%c",&a4,&sep4,&end)==2 && sep4=='%')) &&
          (std::sscanf(st5,"%f%c",&a5,&end)==1 || (std::sscanf(st5,"%f%c%c",&a5,&sep5,&end)==2 && sep5=='%')) &&
          (std::sscanf(st6,"%f%c",&a6,&end)==1 || (std::sscanf(st6,"%f%c%c",&a6,&sep6,&end)==2 && sep6=='%')) &&
          (std::sscanf(st7,"%f%c",&a7,&end)==1 || (std::sscanf(st7,"%f%c%c",&a7,&sep7,&end)==2 && sep7=='%'))) {
        gmic_print("Crop image%s with (%g%s%g%s%g%s%g%s x (%g%s%g%s%g%s%g%s.",gmic_inds,
                   a0,sep0=='%'?"%,":",",a1,sep1=='%'?"%,":",",
                   a2,sep2=='%'?"%,":",",a3,sep3=='%'?"%)":")",
                   a4,sep4=='%'?"%,":",",a5,sep5=='%'?"%,":",",
                   a6,sep6=='%'?"%,":",",a7,sep7=='%'?"%)":")");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            x0 = (int)round(sep0=='%'?a0*img.dimx()/101:a0),
            y0 = (int)round(sep1=='%'?a1*img.dimy()/101:a1),
            z0 = (int)round(sep2=='%'?a2*img.dimz()/101:a2),
            v0 = (int)round(sep3=='%'?a3*img.dimv()/101:a3),
            x1 = (int)round(sep4=='%'?a4*img.dimx()/101:a4),
            y1 = (int)round(sep5=='%'?a5*img.dimy()/101:a5),
            z1 = (int)round(sep6=='%'?a6*img.dimz()/101:a6),
            v1 = (int)round(sep7=='%'?a7*img.dimv()/101:a7);
          img.crop(x0,y0,z0,v0,x1,y1,z1,v1,borders?true:false);
        }
        ++position;
      } else if ((std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                              "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%c",
                              st0,st1,st2,st3,st4,st5,&borders,&end)==7 ||
                  std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                              "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",
                              st0,st1,st2,st3,st4,st5,&end)==6) &&
                 (std::sscanf(st0,"%f%c",&a0,&end)==1 ||
                  (std::sscanf(st0,"%f%c%c",&a0,&sep0,&end)==2 && sep0=='%')) &&
                 (std::sscanf(st1,"%f%c",&a1,&end)==1 ||
                  (std::sscanf(st1,"%f%c%c",&a1,&sep1,&end)==2 && sep1=='%')) &&
                 (std::sscanf(st2,"%f%c",&a2,&end)==1 ||
                  (std::sscanf(st2,"%f%c%c",&a2,&sep2,&end)==2 && sep2=='%')) &&
                 (std::sscanf(st3,"%f%c",&a3,&end)==1 ||
                  (std::sscanf(st3,"%f%c%c",&a3,&sep3,&end)==2 && sep3=='%')) &&
                 (std::sscanf(st4,"%f%c",&a4,&end)==1 ||
                  (std::sscanf(st4,"%f%c%c",&a4,&sep4,&end)==2 && sep4=='%')) &&
                 (std::sscanf(st5,"%f%c",&a5,&end)==1 ||
                  (std::sscanf(st5,"%f%c%c",&a5,&sep5,&end)==2 && sep5=='%'))) {
        gmic_print("Crop image%s with (%g%s%g%s%g%s x (%g%s%g%s%g%s.",gmic_inds,
                   a0,sep0=='%'?"%,":",",a1,sep1=='%'?"%,":",",a2,sep2=='%'?"%)":")",
                   a3,sep3=='%'?"%,":",",a4,sep4=='%'?"%,":",",a5,sep5=='%'?"%)":")");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            x0 = (int)round(sep0=='%'?a0*img.dimx()/101:a0),
            y0 = (int)round(sep1=='%'?a1*img.dimy()/101:a1),
            z0 = (int)round(sep2=='%'?a2*img.dimz()/101:a2),
            x1 = (int)round(sep3=='%'?a3*img.dimx()/101:a3),
            y1 = (int)round(sep4=='%'?a4*img.dimy()/101:a4),
            z1 = (int)round(sep5=='%'?a5*img.dimz()/101:a5);
          img.crop(x0,y0,z0,x1,y1,z1,borders?true:false);
        }
        ++position;
      } else if ((std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                              "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%c",
                              st0,st1,st2,st3,&borders,&end)==5 ||
                  std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                              "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",
                              st0,st1,st2,st3,&end)==4) &&
                 (std::sscanf(st0,"%f%c",&a0,&end)==1 ||
                  (std::sscanf(st0,"%f%c%c",&a0,&sep0,&end)==2 && sep0=='%')) &&
                 (std::sscanf(st1,"%f%c",&a1,&end)==1 ||
                  (std::sscanf(st1,"%f%c%c",&a1,&sep1,&end)==2 && sep1=='%')) &&
                 (std::sscanf(st2,"%f%c",&a2,&end)==1 ||
                  (std::sscanf(st2,"%f%c%c",&a2,&sep2,&end)==2 && sep2=='%')) &&
                 (std::sscanf(st3,"%f%c",&a3,&end)==1 ||
                  (std::sscanf(st3,"%f%c%c",&a3,&sep3,&end)==2 && sep3=='%'))) {
        gmic_print("Crop image%s with (%g%s%g%s x (%g%s%g%s.",gmic_inds,
                   a0,sep0=='%'?"%,":",",a1,sep1=='%'?"%)":")",
                   a2,sep2=='%'?"%,":",",a3,sep3=='%'?"%)":")");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            x0 = (int)round(sep0=='%'?a0*img.dimx()/101:a0),
            y0 = (int)round(sep1=='%'?a1*img.dimy()/101:a1),
            x1 = (int)round(sep2=='%'?a2*img.dimx()/101:a2),
            y1 = (int)round(sep3=='%'?a3*img.dimy()/101:a3);
          img.crop(x0,y0,x1,y1,borders?true:false);
        }
        ++position;
      } else if ((std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%c",st0,st1,&borders,&end)==3 ||
                  std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",st0,st1,&end)==2) &&
                 (std::sscanf(st0,"%f%c",&a0,&end)==1 ||
                  (std::sscanf(st0,"%f%c%c",&a0,&sep0,&end)==2 && sep0=='%')) &&
                 (std::sscanf(st1,"%f%c",&a1,&end)==1 ||
                  (std::sscanf(st1,"%f%c%c",&a1,&sep1,&end)==2 && sep1=='%'))) {
        gmic_print("Crop image%s with (%g%s x (g%s.",gmic_inds,
                   a0,sep0=='%'?"%)":")",a1,sep1=='%'?"%)":")");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            x0 = (int)round(sep0=='%'?a0*img.dimx()/101:a0),
            x1 = (int)round(sep1=='%'?a1*img.dimx()/101:a1);
          img.crop(x0,x1,borders?true:false);
        }
        ++position;
      } else {
        gmic_print("Crop image%s : Interactive mode.",gmic_inds);
        char title[1024] = { 0 };
        cimg_foroff(indices,l) {
          CImg<T>& img = images[indices[l]];
          CImgDisplay disp(cimg_fitscreen(img.dimx(),img.dimy(),1),0,1);
          std::sprintf(title,"%s : Interactive crop",filenames[indices[l]].ptr());
          disp.set_title(title);
          const CImg<int> s = img.get_user_coordinates(2,disp);
          gmic_print("Crop image [%d] with (%d,%d,%d) x (%d,%d,%d).",
                     indices[l],s[0],s[1],s[2],s[3],s[4],s[5]);
          img.crop(s[0],s[1],s[2],s[3],s[4],s[5]);
        }
      }
      continue;
    }

    // Channels.
    if (!cimg::strcmp("-channels",item0)) {
      char sep0 = 0, sep1 = 0, end = 0, arg0[256] = { 0 }, arg1[256] = { 0 };
      float value0 = 0, value1 = 0; int ind0 = no_ind, ind1 = no_ind;
      if (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",arg0,arg1,&end)==2 &&
	  ((std::sscanf(arg0,"[%d%c%c]",&ind0,&sep0,&end)==2 && sep0==']') ||
	   (std::sscanf(arg0,"%f%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
	   std::sscanf(arg0,"%f%c",&value0,&end)==1) &&
	  ((std::sscanf(arg1,"[%d%c%c]",&ind1,&sep1,&end)==2 && sep1==']') ||
	   (std::sscanf(arg1,"%f%c%c",&value1,&sep1,&end)==2 && sep1=='%') ||
	   std::sscanf(arg1,"%f%c",&value1,&end)==1)) {
	if (ind0!=no_ind) { gmic_check_indice(ind0,"Keep channels of image%s"); value0 = images[ind0].dimv()-1.0f; sep0 = 0; }
        if (ind1!=no_ind) { gmic_check_indice(ind1,"Keep channels of image%s"); value1 = images[ind1].dimv()-1.0f; sep1 = 0; }
        gmic_print("Keep channels %g%s..%g%s of image%s.",value0,sep0=='%'?"%":"",value1,sep1=='%'?"%":"",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            nvalue0 = (int)round(sep0=='%'?value0*img.dimv()/101:value0),
            nvalue1 = (int)round(sep1=='%'?value1*img.dimv()/101:value1);
          img.channels(nvalue0,nvalue1);
        }
      } else if (std::sscanf(argument,"%255[][0-9.eE%+-]%c",arg0,&end)==1 &&
		 ((std::sscanf(arg0,"[%d%c%c]",&ind0,&sep0,&end)==2 && sep0==']') ||
		  (std::sscanf(arg0,"%f%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
		  std::sscanf(arg0,"%f%c",&value0,&end)==1)) {
	if (ind0!=no_ind) { gmic_check_indice(ind0,"Keep channel of image%s"); value0 = images[ind0].dimv()-1.0f; sep0 = 0; }
        gmic_print("Keep channel %g%s of image%s.",value0,sep0=='%'?"%":"",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int nvalue0 = (int)round(sep0=='%'?value0*img.dimv()/101:value0);
          img.channel(nvalue0);
        }
      } else gmic_error("Keep channels of image%s : Invalid argument '%s' "
                        "(should be 'channel0[%][,channel1[%]]').",gmic_inds,argument);
      ++position; continue;
    }

    // Slices.
    if (!cimg::strcmp("-slices",item0)) {
      char sep0 = 0, sep1 = 0, end = 0, arg0[256] = { 0 }, arg1[256] = { 0 };
      float value0 = 0, value1 = 0; int ind0 = no_ind, ind1 = no_ind;
      if (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",arg0,arg1,&end)==2 &&
	  ((std::sscanf(arg0,"[%d%c%c]",&ind0,&sep0,&end)==2 && sep0==']') ||
	   (std::sscanf(arg0,"%f%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
	   std::sscanf(arg0,"%f%c",&value0,&end)==1) &&
	  ((std::sscanf(arg1,"[%d%c%c]",&ind1,&sep1,&end)==2 && sep1==']') ||
	   (std::sscanf(arg1,"%f%c%c",&value1,&sep1,&end)==2 && sep1=='%') ||
	   std::sscanf(arg1,"%f%c",&value1,&end)==1)) {
	if (ind0!=no_ind) { gmic_check_indice(ind0,"Keep slices of image%s"); value0 = images[ind0].dimv()-1.0f; sep0 = 0; }
        if (ind1!=no_ind) { gmic_check_indice(ind1,"Keep slices of image%s"); value1 = images[ind1].dimv()-1.0f; sep1 = 0; }
        gmic_print("Keep slices %g%s..%g%s of image%s.",value0,sep0=='%'?"%":"",value1,sep1=='%'?"%":"",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            nvalue0 = (int)round(sep0=='%'?value0*img.dimv()/101:value0),
            nvalue1 = (int)round(sep1=='%'?value1*img.dimv()/101:value1);
          img.slices(nvalue0,nvalue1);
        }
      } else if (std::sscanf(argument,"%255[][0-9.eE%+-]%c",arg0,&end)==1 &&
		 ((std::sscanf(arg0,"[%d%c%c]",&ind0,&sep0,&end)==2 && sep0==']') ||
		  (std::sscanf(arg0,"%f%c%c",&value0,&sep0,&end)==2 && sep0=='%') ||
		  std::sscanf(arg0,"%f%c",&value0,&end)==1)) {
	if (ind0!=no_ind) { gmic_check_indice(ind0,"Keep slice of image%s"); value0 = images[ind0].dimv()-1.0f; sep0 = 0; }
        gmic_print("Keep slice %g%s of image%s.",value0,sep0=='%'?"%":"",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int nvalue0 = (int)round(sep0=='%'?value0*img.dimv()/101:value0);
          img.slice(nvalue0);
        }
      } else gmic_error("Keep slices of image%s : Invalid argument '%s' "
                        "(should be 'slice0[%][,slice1[%]]').",gmic_inds,argument);
      ++position; continue;
    }

    // Rotate.
    if (!cimg::strcmp("-rotate",item0)) {
      float angle = 0; int borders = 3; char end = 0;
      if (std::sscanf(argument,"%f%*c%d%c",&angle,&borders,&end)==2 ||
          std::sscanf(argument,"%f%c",&angle,&end)==1) {
        gmic_print("Rotate image%s with an angle of %g deg.",gmic_inds,angle);
        cimg_foroff(indices,l) images[indices[l]].rotate(angle,borders);
        ++position;
      } else gmic_error("Rotate image%s : Invalid argument '%s' "
                        "(should be 'angle[,border_conditions]').",gmic_inds,argument);
      continue;
    }

    // Mirror.
    if (!cimg::strcmp("-mirror",item0)) {
      const char axis = cimg::uncase(*argument);
      if (cimg::strlen(argument)==1 && (axis=='x' || axis=='y' || axis=='z' || axis=='v')) {
        gmic_print("Mirror image%s along the %c-axis.",gmic_inds,axis);
        cimg_foroff(indices,l) images[indices[l]].mirror(axis);
      } else gmic_error("Mirror image%s : Invalid argument '%s' "
                        "(should be '{x,y,z,v}').",gmic_inds,argument);
      ++position; continue;
    }

    // Translate.
    if (!cimg::strcmp("-translate",item0)) {
      char stx[256] = { 0 }, sty[256] = { 0 }, stz[256] = { 0 }, stv[256] = { 0 };
      char sepx = 0, sepy = 0, sepz = 0, sepv = 0, end = 0;
      float dx = 0, dy = 0, dz = 0, dv = 0; int borders = 0;
      if (((std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%c",
                        stx,sty,stz,stv,&borders,&end)==5 ||
            std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",
                        stx,sty,stz,stv,&end)==4) &&
           (std::sscanf(stx,"%f%c",&dx,&end)==1 || (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%')) &&
           (std::sscanf(sty,"%f%c",&dy,&end)==1 || (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%')) &&
           (std::sscanf(stz,"%f%c",&dz,&end)==1 || (std::sscanf(stz,"%f%c%c",&dz,&sepz,&end)==2 && sepz=='%')) &&
           (std::sscanf(stv,"%f%c",&dv,&end)==1 || (std::sscanf(stv,"%f%c%c",&dv,&sepv,&end)==2 && sepv=='%'))) ||
          (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",stx,sty,stz,&end)==3 &&
           (std::sscanf(stx,"%f%c",&dx,&end)==1 || (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%')) &&
           (std::sscanf(sty,"%f%c",&dy,&end)==1 || (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%')) &&
           (std::sscanf(stz,"%f%c",&dz,&end)==1 || (std::sscanf(stz,"%f%c%c",&dz,&sepz,&end)==2 && sepz=='%'))) ||
          (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%c",stx,sty,&end)==2 &&
           (std::sscanf(stx,"%f%c",&dx,&end)==1 || (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%')) &&
           (std::sscanf(sty,"%f%c",&dy,&end)==1 || (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%'))) ||
          (std::sscanf(argument,"%255[0-9.eE%+-]%c",stx,&end)==1 &&
           (std::sscanf(stx,"%f%c",&dx,&end)==1 || (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%')))) {
        gmic_print("Translate image%s with vector (%g%s,%g%s,%g%s,%g%s).",
                   gmic_inds,dx,sepx=='%'?"%":"",dy,sepy=='%'?"%":"",dz,sepz=='%'?"%":"",dv,sepv=='%'?"%":"");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            ndx = (int)round(sepx=='%'?dx*img.dimx()/100:dx),
            ndy = (int)round(sepy=='%'?dy*img.dimy()/100:dy),
            ndz = (int)round(sepz=='%'?dz*img.dimz()/100:dz),
            ndv = (int)round(sepv=='%'?dv*img.dimv()/100:dv);
          images[indices[l]].translate(ndx,ndy,ndz,ndv,borders);
        }
      } else gmic_error("Translate image%s : Invalid argument '%s' "
                        "(should be 'tx[%][,ty[%][,tz[%][,tv[%][,border_conditions]]]]').",gmic_inds,argument);
      ++position; continue;
    }

    // Transpose.
    gmic_simple_item("-transpose",transpose,"Transpose image%s.");

    // Permute axes.
    if (!cimg::strcmp("-permute",item0)) {
      gmic_print("Permute axes of image%s with permutation '%s'.",gmic_inds,argument);
      cimg_foroff(indices,l) images[indices[l]].permute_axes(argument);
      ++position; continue;
    }

    // Unroll.
    if (!cimg::strcmp("-unroll",item0)) {
      const char axis = cimg::uncase(*argument);
      if (cimg::strlen(argument)==1 && (axis=='x' || axis=='y' || axis=='z' || axis=='v')) {
        gmic_print("Unroll image%s along the %c-axis.",gmic_inds,axis);
        cimg_foroff(indices,l) images[indices[l]].unroll(axis);
      } else gmic_error("Unroll image%s : Invalid argument '%s' "
                        "(should be '{x,y,z,v}').",gmic_inds,argument);
      ++position; continue;
    }

    // Split image(s).
    if (!cimg::strcmp("-split",item0) || !cimg::strcmp("-s",item0)) {
      char axis = cimg::uncase(*argument), foo = 0, end = 0; int nb = 0, keep_value = 0; double value = 0;
      if ((std::sscanf(argument,"%c%*c%d%c",&foo,&nb,&end)==2 ||
           std::sscanf(argument,"%c%c",&foo,&end)==1) &&
          (axis=='x' || axis=='y' || axis=='z' || axis=='v')) {
        if (nb<0) gmic_error("Split image%s along the %c-axis in %d part : Invalid number of parts.",
                             gmic_inds,axis,nb);
        if (nb>0) gmic_print("Split image%s along the %c-axis in %d parts.",gmic_inds,axis,nb);
        else gmic_print("Split image%s along the %c-axis.",gmic_inds,axis);
        unsigned int off = 0;
        cimg_foroff(indices,l) {
          const unsigned int ind = indices[l] + off;
          CImg<T>& img = images[ind];
          const CImg<char> filename = filenames[ind];
          const CImgList<T> split = img.get_split(axis,nb);
          images.remove(ind); images.insert(split,ind);
          filenames.remove(ind); filenames.insert(split.size,filename,ind);
          off+=split.size-1;
        }
      } else if (std::sscanf(argument,"%lf%*c%d%c",&value,&keep_value,&end)==2 ||
                 std::sscanf(argument,"%lf%c",&value,&end)==1) {
        gmic_print("Split image%s according to value %g.",gmic_inds,value);
        unsigned int off = 0;
        cimg_foroff(indices,l) {
          const unsigned int ind = indices[l] + off;
          CImg<T>& img = images[ind];
          const CImg<char> filename = filenames[ind];
          const CImgList<T> split = img.get_split((T)value,keep_value,false);
          images.remove(ind); images.insert(split,ind);
          filenames.remove(ind); filenames.insert(split.size,filename,ind);
          off+=split.size-1;
        }
      } else gmic_error("Split image%s : Invalid argument '%s' "
                        "(should be 'axis[,nb_parts]' where 'axis' can be '{x,y,z,v}').",gmic_inds,argument);
      ++position; continue;
    }

    // Append image(s).
    if (!cimg::strcmp("-append",item0) || !cimg::strcmp("-a",item0)) {
      char axis = 0, align='p', end = 0;
      if (std::sscanf(argument,"%c%*c%c%c",&axis,&align,&end)==2 ||
          std::sscanf(argument,"%c%c",&axis,&end)==1) {
        axis = cimg::uncase(axis);
        if (axis!='x' && axis!='y' && axis!='z' && axis!='v')
          gmic_error("Append image%s : Invalid axis '%s'.",gmic_inds,argument);
        if (align!='p' && align!='c' && align!='n')
          gmic_error("Append image%s : Invalid alignment '%c'.",gmic_inds,align);
        gmic_print("Append image%s along the %c-axis with %s alignment.",
                   gmic_inds,axis,align=='p'?"left":align=='c'?"center":"right");
        CImgList<T> subimages; cimg_foroff(indices,l) subimages.insert(images[indices[l]],~0U,true);
        images.insert(subimages.get_append(axis,align),indices[0]);
        filenames.insert(filenames[indices[0]],indices[0]);
        int off = 1;
        cimg_foroff(indices,l) {
          const int ind = indices[l] + off;
          images.remove(ind); filenames.remove(ind);
          --off;
        }
      } else gmic_error("Append image%s : Invalid argument '%s' "
                        "(should be 'axis[,alignement]' where 'axis' can be '{x,y,z,v}' "
                        "and alignement '{p,c,n}').",gmic_inds,argument);
      ++position; continue;
    }

    // Warp image(s).
    if (!cimg::strcmp("-warp",item0)) {
      int ind0 = 0, interpolation = 1, relative = 0, borders = 1, nb = 1; char end = 0, sep = 0;
      if (std::sscanf(argument,"[%d]%*c%d%*c%d%*c%d%*c%d%c",
                      &ind0,&nb,&relative,&interpolation,&borders,&end)==5 ||
          std::sscanf(argument,"[%d]%*c%d%*c%d%*c%d%c",&ind0,&nb,&relative,&interpolation,&end)==4 ||
          std::sscanf(argument,"[%d]%*c%d%*c%d%c",&ind0,&nb,&relative,&end)==3 ||
          std::sscanf(argument,"[%d]%*c%d%c",&ind0,&nb,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Warp image%s");
        if (nb!=1) gmic_print("Warp image%s with %s field [%u] and %d frames.",
                              gmic_inds,relative?"relative":"absolute",ind0,nb);
        else gmic_print("Warp image%s with %s field [%u].",gmic_inds,relative?"relative":"absolute",ind0);
        if (nb>=1) {
          const CImg<T> warp = images[ind0];
          unsigned int off = 0;
          cimg_foroff(indices,l) {
            const unsigned int ind = indices[l] + off;
            CImg<T> &img = images[ind];
            CImgList<T> frames(nb);
            cimglist_for(frames,t) {
              const CImg<T> nwarp = warp.get_resize(img.dimx(),img.dimy(),img.dimz(),warp.dimv(),3)*=(t+1.0f)/nb;
              frames[t] = img.get_warp(nwarp,relative?true:false,interpolation?true:false,borders?true:false);
            }
            images.remove(ind); images.insert(frames,ind);
            filenames.insert(nb-1,filenames[ind],ind);
            off+=nb-1;
          }
        }
      } else gmic_error("Warp image%s : Invalid argument '%s' "
                        "(should be '[indice][,interpolation[,border_conditions]]').",gmic_inds,argument);
      ++position; continue;
    }

    //-----------------------
    // Image filtering
    //-----------------------

    // Gaussian blur.
    if (!cimg::strcmp("-blur",item0)) {
      float sigma = -1; int borders = 1; char end = 0;
      if ((std::sscanf(argument,"%f%*c%d%c",&sigma,&borders,&end)==2 ||
           std::sscanf(argument,"%f%c",&sigma,&end)==1)
          && sigma>=0) {
        gmic_print("Blur image%s with standard deviation %g.",gmic_inds,sigma);
        cimg_foroff(indices,l) images[indices[l]].blur(sigma,borders?true:false);
      } else gmic_error("Blur image%s : Invalid argument '%s' "
                        "(should be 'stdev[,border_conditions]', with stdev>=0).",gmic_inds,argument);
      ++position; continue;
    }

    // Bilateral filter.
    if (!cimg::strcmp("-bilateral",item0)) {
      float sigmas = 0, sigmar = 0; char end = 0;
      if (std::sscanf(argument,"%f%*c%f%c",&sigmas,&sigmar,&end)==2) {
        if (sigmas<0 || sigmar<0)
          gmic_error("Apply bilateral filter on image%s with standart deviations %g and %g : "
                     "Invalid standard deviations.",gmic_inds,sigmas,sigmar);
        gmic_print("Apply bilateral filter on image%s with standart deviations %g and %g.",
                   gmic_inds,sigmas,sigmar);
        cimg_foroff(indices,l) images[indices[l]].blur_bilateral(sigmas,sigmar);
      } else gmic_error("Apply bilateral filter on image%s : Invalid argument '%s' "
                        "(should be 'stdevs,stdevr').",gmic_inds,argument);
      ++position; continue;
    }

    // Smooth.
    if (!cimg::strcmp("-smooth",item0)) {
      float amplitude = 0, sharpness = 0.7f, anisotropy = 0.3f, alpha = 0.6f, sigma = 1.1f; char end = 0;
      if (std::sscanf(argument,"%f%*c%f%*c%f%*c%f%*c%f%c",
                      &amplitude,&sharpness,&anisotropy,&alpha,&sigma,&end)==5 ||
          std::sscanf(argument,"%f%*c%f%*c%f%*c%f%c",&amplitude,&sharpness,&anisotropy,&alpha,&end)==4 ||
          std::sscanf(argument,"%f%*c%f%*c%f%c",&amplitude,&sharpness,&anisotropy,&end)==3 ||
          std::sscanf(argument,"%f%*c%f%c",&amplitude,&sharpness,&end)==2 ||
          std::sscanf(argument,"%f%c",&amplitude,&end)==1) {
        if (amplitude<0 || sharpness<0 || anisotropy<0 || anisotropy>1 || alpha<0 || sigma<0)
          gmic_error("Smooth image%s anisotropically with "
                     "amplitude %g, sharpness %g, anisotropy %g, alpha %g and sigma %g : Invalid parameters.",
                     gmic_inds,amplitude,sharpness,anisotropy,alpha,sigma);
        gmic_print("Smooth image%s anisotropically with "
                   "amplitude %g, sharpness %g, anisotropy %g, alpha %g and sigma %g.",
                   gmic_inds,amplitude,sharpness,anisotropy,alpha,sigma);
        cimg_foroff(indices,l)
          images[indices[l]].blur_anisotropic(amplitude,sharpness,anisotropy,alpha,sigma);
      } else gmic_error("Smooth image%s anisotropically : Invalid argument '%s' "
                        "(should be 'amplitude[,sharpness[,anisotropy[,alpha[,sigma]]]]').",
                        gmic_inds,argument);
      ++position; continue;
    }

    // Patch averaging.
    if (!cimg::strcmp("-denoise",item0)) {
      float sigmas = 10, sigmar = 10; int psize = 5, rsize = 6; char end = 0;
      if (std::sscanf(argument,"%f%*c%f%*c%d%*c%d%c",&sigmas,&sigmar,&psize,&rsize,&end)==4 ||
          std::sscanf(argument,"%f%*c%f%*c%d%c",&sigmas,&sigmar,&psize,&end)==3 ||
          std::sscanf(argument,"%f%*c%f%c",&sigmas,&sigmar,&end)==2 ||
          std::sscanf(argument,"%f%c",&sigmas,&end)==1) {
        if (sigmas<0 || sigmar<0 || psize<0 || rsize<0)
          gmic_print("Denoise image%s with %dx%d patches, standard deviations %lg,%g and lookup size %d : "
                     "Invalid parameters.",gmic_inds,psize,psize,sigmas,sigmar,rsize);
        gmic_print("Denoise image%s with %dx%d patches, standard deviations %lg,%g and lookup size %d.",
                   gmic_inds,psize,psize,sigmas,sigmar,rsize);
        cimg_foroff(indices,l) images[indices[l]].blur_patch(psize,sigmas,sigmar,rsize);
      } else gmic_error("Denoise image%s : Invalid argument '%s' "
                        "(should be 'stdev1[,stdev2[,patch_size[,lookup_size]]]').",
                        gmic_inds,argument);
      ++position; continue;
    }

    // Median filter.
    if (!cimg::strcmp("-median",item0)) {
      int siz = 3; char end = 0;
      if (std::sscanf(argument,"%d%c",&siz,&end)==1) ++position;
      if (siz<=0) gmic_error("Apply median filter of size %d on image%s : Invalid size.",siz,gmic_inds);
      gmic_print("Apply median filter of size %d on image%s.",siz,gmic_inds);
      cimg_foroff(indices,l) images[indices[l]].blur_median(siz);
      continue;
    }

    // Sharpen.
    if (!cimg::strcmp("-sharpen",item0)) {
      float amplitude = 0, edge = 1, alpha = 0, sigma = 0; int sharpen_type = 0; char end = 0;
      if (std::sscanf(argument,"%f%*c%d%*c%f%*c%f%*c%f%c",&amplitude,&sharpen_type,&edge,&alpha,&sigma,&end)==5 ||
          std::sscanf(argument,"%f%*c%d%*c%f%*c%f%c",&amplitude,&sharpen_type,&edge,&alpha,&end)==4 ||
          std::sscanf(argument,"%f%*c%d%*c%f%c",&amplitude,&sharpen_type,&edge,&end)==3 ||
          std::sscanf(argument,"%f%*c%d%c",&amplitude,&sharpen_type,&end)==2 ||
          std::sscanf(argument,"%f%c",&amplitude,&end)==1) {
        if (amplitude<0 || edge<0 || alpha<0 || sigma<0)
          gmic_error("Sharpen image%s with amplitude %g, edge %g, alpha %g and sigma %g : Invalid parameters.",
                     gmic_inds,amplitude,edge,alpha,sigma);
        if (sharpen_type)
          gmic_print("Sharpen image%s with shock filters and amplitude %g, edge %g, alpha %g and sigma %g.",
                     gmic_inds,amplitude,edge,alpha,sigma);
        else
          gmic_print("Sharpen image%s with inverse diffusion and amplitude %g.",gmic_inds,amplitude);
        cimg_foroff(indices,l) images[indices[l]].sharpen(amplitude,sharpen_type?true:false,edge,alpha,sigma);
      } else gmic_error("Sharpen image%s : Invalid argument '%s' "
                        "(should be 'amplitude[,sharpen_type[,edge[,alpha[,sigma]]]]', "
                        "where 'sharpen_type' can be '{0=inverse diffusion, 1=shock filters}').",
                        gmic_inds,argument);
      ++position; continue;
    }

    // Convolve.
    if (!cimg::strcmp("-convolve",item0)) {
      int ind0 = 0, borders = 1; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d]%*c%d%c",&ind0,&borders,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Convolve image%s");
        gmic_print("Convolve image%s with mask [%d].",gmic_inds,ind0);
        const CImg<T> mask = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].convolve(mask,borders);
      } else gmic_error("Convolve image%s : Invalid argument '%s' "
                        "(should be '[indice][,border_conditions]').",gmic_inds,argument);
      ++position; continue;
    }

    // Correlate.
    if (!cimg::strcmp("-correlate",item0)) {
      int ind0 = 0, borders = 1; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d]%*c%d%c",&ind0,&borders,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Correlate image%s");
        gmic_print("Correlate image%s with mask [%d].",gmic_inds,ind0);
        const CImg<T> mask = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].correlate(mask,borders);
      } else gmic_error("Correlate image%s : Invalid argument '%s' "
                        "(should be '[indice][,border_conditions]').",gmic_inds,argument);
      ++position; continue;
    }

    // Erode.
    if (!cimg::strcmp("-erode",item0)) {
      int siz = 3, ind0 = 0, borders = 1; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d]%*c%d%c",&ind0,&borders,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Erode image%s");
        gmic_print("Erode image%s with mask [%d].",gmic_inds,ind0);
        const CImg<T> mask = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].erode(mask,borders);
        ++position;
      } else {
        if (std::sscanf(argument,"%d%*c%d%c",&siz,&borders,&end)==2 ||
            std::sscanf(argument,"%d%c",&siz,&end)==1) ++position;
        if (siz<=0) gmic_error("Erode image%s with erosion size %d : Invalid size.",gmic_inds,siz);
        gmic_print("Erode image%s with erosion size %d.",gmic_inds,siz);
        cimg_foroff(indices,l) images[indices[l]].erode(siz,borders);
      }
      continue;
    }

    // Dilate.
    if (!cimg::strcmp("-dilate",item0)) {
      int siz = 3, ind0 = 0, borders = 1; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d]%*c%d%c",&ind0,&borders,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Dilate image%s");
        gmic_print("Dilate image%s with mask [%d].",gmic_inds,ind0);
        const CImg<T> mask = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].dilate(mask,borders);
        ++position;
      } else {
        if (std::sscanf(argument,"%d%*c%d%c",&siz,&borders,&end)==2 ||
            std::sscanf(argument,"%d%c",&siz,&end)==1) ++position;
        if (siz<=0) gmic_error("Dilate image%s with dilatation size %d : Invalid size.",gmic_inds,siz);
        gmic_print("Dilate image%s with dilatation size %d.",gmic_inds,siz);
        cimg_foroff(indices,l) images[indices[l]].dilate(siz,borders);
      }
      continue;
    }

    // Compute gradient.
    if (!cimg::strcmp("-gradient",item0)) {
      char axes[256] = { 0 }, *naxes = 0, end = 0; int scheme = 3;
      gmic_print("Compute gradient of image%s.",gmic_inds);
      if (std::sscanf(argument,"%255[xyz]%*c%d%c",axes,&scheme,&end)==2 ||
          std::sscanf(argument,"%255[xyz]%c",axes,&end)==1) { naxes = axes; ++position; }
      unsigned int off = 0;
      cimg_foroff(indices,l) {
        const unsigned int ind = indices[l] + off;
        CImg<T>& img = images[ind];
        const CImg<char> filename = filenames[ind];
        const CImgList<T> gradient = img.get_gradient(naxes,scheme);
        images.remove(ind); images.insert(gradient,ind);
        filenames.remove(ind); filenames.insert(gradient.size,filename,ind);
        off+=gradient.size-1;
      }
      continue;
    }

    // Compute Hessian.
    if (!cimg::strcmp("-hessian",item0)) {
      gmic_print("Compute Hessian of image%s.",gmic_inds);
      unsigned int off = 0;
      cimg_foroff(indices,l) {
        const unsigned int ind = indices[l] + off;
        CImg<T>& img = images[ind];
        const CImg<char> filename = filenames[ind];
        const CImgList<T> gradient = img.get_hessian();
        images.remove(ind); images.insert(gradient,ind);
        filenames.remove(ind); filenames.insert(gradient.size,filename,ind);
        off+=gradient.size-1;
      }
      continue;
    }

    // Compute direct or inverse FFT.
    const bool inv_fft = !cimg::strcmp("-ifft",item0);
    if (!cimg::strcmp("-fft",item0) || inv_fft) {
      gmic_print("Compute %sFourier Transform of complex data",inv_fft?"inverse ":"");
      cimg_foroff(indices,l) {
        const unsigned int ind0 = indices[l], ind1 = l+1<_maxl?indices[l+1]:~0U;
        if (ind1!=~0U) {
          if (!quiet) std::fprintf(stderr," ([%u],[%u])%c",ind0,ind1,l==_maxl-1?'.':',');
          CImgList<T> fft(images[ind0],images[ind1],true);
          fft.FFT(inv_fft);
          fft[0].transfer_to(images[indices[0]]);
          fft[1].transfer_to(images[indices[1]]);
          ++l;
        } else {
          if (!quiet) std::fprintf(stderr," ([%u],0)",ind0);
          CImgList<T> fft(images[ind0],true);
          fft.insert(fft[0],~0U,false);
          fft[1].fill(0);
          fft.FFT(inv_fft);
          fft[0].transfer_to(images[ind0]);
          images.insert(fft[1],1+ind0);
          filenames.insert(filenames[ind0],1+ind0);
        }
      }
      continue;
    }

    //-----------------------------
    // Image creation and drawing
    //-----------------------------

    // Dimensions.
    if (!cimg::strcmp("-dimensions",item0)) {
      gmic_print("Get dimensions of image%s.",gmic_inds);
      cimg_foroff(indices,l) {
        CImg<T>& img = images[indices[l]];
        img = CImg<T>::vector((T)img.width,(T)img.height,(T)img.depth,(T)img.dim);
      }
      continue;
    }

    // Stats.
    if (!cimg::strcmp("-stats",item0)) {
      gmic_print("Get statistics of image%s.",gmic_inds);
      cimg_foroff(indices,l) images[indices[l]].stats();
      continue;
    }

    // Histogram.
    if (!cimg::strcmp("-histogram",item0)) {
      int nb_levels = 256; char sep = 0, end = 0;
      if ((std::sscanf(argument,"%d%c%c",&nb_levels,&sep,&end)==2 && sep=='%') ||
          std::sscanf(argument,"%d%c",&nb_levels,&end)==1) ++position;
      else { nb_levels = 100; sep = '%'; }
      gmic_print("Compute histogram of image%s using %d%s levels.",gmic_inds,nb_levels,sep=='%'?"%":"");
      cimg_foroff(indices,l) {
        CImg<T> &img = images[indices[l]];
        int nnb_levels = nb_levels;
        if (sep=='%') { double m, M = img.maxmin(m); nnb_levels = (int)round(nb_levels*(1+M-m)/100); }
        images[indices[l]].histogram(nnb_levels);
      }
      continue;
    }

    // Distance function.
    if (!cimg::strcmp("-distance",item0)) {
      double value = 0; char sep = 0, end = 0;
      if ((std::sscanf(argument,"%lf%c%c",&value,&sep,&end)==2 && sep=='%') ||
          std::sscanf(argument,"%lf%c",&value,&end)==1) {
        gmic_print("Compute distance map of image%s to isovalue %g%s.",gmic_inds,value,sep=='%'?"%":"");
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          double isovalue = value;
          if (sep=='%') { double m, M = img.maxmin(m); isovalue = m + value*(M - m)/100; }
          images[indices[l]].distance((T)isovalue);
        }
      } else gmic_error("Compute distance function of image%s : Invalid argument '%s' "
                        "(should be 'value[%]').",gmic_inds,argument);
      ++position; continue;
    }

    // Label regions.
    gmic_simple_item("-label",label_regions,"Label regions on image%s.");

    // Displacement field.
    if (!cimg::strcmp("-displacement",item0)) {
      float smooth = 0.1f; int ind0 = 0; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d]%*c%f%c",&ind0,&smooth,&end)==2 ||
          (std::sscanf(argument,"[%d%c%c]",&ind0,&sep,&end)==2 && sep==']')) {
	gmic_check_indice(ind0,"Compute displacement field of image%s");
        gmic_print("Compute displacement field of image%s with target [%u] and smoothness %g.",
                   gmic_inds,ind0,smooth);
        const CImg<T> target = images[ind0];
        cimg_foroff(indices,l) images[indices[l]].displacement_field(target,smooth);
      } else gmic_error("Compute displacement field of image%s : Invalid argument '%s' "
                        "(should be '[indice][,smoothness]').",gmic_inds,argument);
      ++position; continue;
    }

    // Sort.
    gmic_simple_item("-sort",sort,"Sort values in image%s.");

    // PSNR.
    if (!cimg::strcmp("-psnr",item0)) {
      double valmax = 255; char end = 0;
      if (std::sscanf(argument,"%lf%c",&valmax,&end)==1) ++position;
      if (images.size) {
        const unsigned int siz = indices.size();
        gmic_print("Compute a %ux%u matrix [%u] of PSNR values (max. pixel value is %g).",siz,siz,images.size,valmax);
        CImg<T> res(siz,siz,1,1,(T)-1);
        cimg_forXY(res,x,y) if (x>y) res(x,y) = res(y,x) = (T)images[indices[x]].PSNR(images[indices[y]],valmax);
        images.insert(res);
        filenames.insert(CImg<char>("PSNR",5,1,1,1,false));
      } else gmic_error("Compute PSNR : No images available.");
      continue;
    }

    // Draw point.
    if (!cimg::strcmp("-point",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, color[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, sepz0 = 0, end = 0;
      float x0 = 0, y0 = 0, z0 = 0;
      if (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%1023[0-9.eE,+-]",
                      arg0,arg1,arg2,color)>=2 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1) &&
          ((std::sscanf(arg2,"%f%c%c",&z0,&sepz0,&end)==2 && sepz0=='%') ||
           std::sscanf(arg2,"%f%c",&z0,&end)==1 || !arg2[0])) {
        gmic_print("Draw point (%g%s,%g%s,%g%s) with color '%s' on image%s.",
                   x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",z0,sepz0=='%'?"%":"",
                   color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0),
            nz0 = (int)round(sepz0=='%'?z0*img.depth/101:z0);
          img.draw_point(nx0,ny0,nz0,col);
        }
      } else gmic_error("Draw point on image%s : Invalid argument '%s' "
                        "(should be 'x[%],y[%][,z[%][,color]])",gmic_inds,argument);
      ++position; continue;
    }

    // Draw line.
    if (!cimg::strcmp("-line",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, arg3[256] = { 0 }, color[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, sepx1 = 0, sepy1 = 0, end = 0;
      float x0 = 0, y0 = 0, x1 = 0, y1 = 0;
      if (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%1023[0-9.eE,+-]",
                      arg0,arg1,arg2,arg3,color)>=4 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1) &&
          ((std::sscanf(arg2,"%f%c%c",&x1,&sepx1,&end)==2 && sepx1=='%') ||
           std::sscanf(arg2,"%f%c",&x1,&end)==1) &&
          ((std::sscanf(arg3,"%f%c%c",&y1,&sepy1,&end)==2 && sepy1=='%') ||
           std::sscanf(arg3,"%f%c",&y1,&end)==1)) {
        gmic_print("Draw line (%g%s,%g%s) - (%g%s,%g%s) with color '%s' on image%s.",
                   x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",x1,sepx1=='%'?"%":"",y1,sepy1=='%'?"%":"",
                   color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0),
            nx1 = (int)round(sepx1=='%'?x1*img.width/101:x1),
            ny1 = (int)round(sepy1=='%'?y1*img.height/101:y1);
          img.draw_line(nx0,ny0,nx1,ny1,col);
        }
      } else gmic_error("Draw line on image%s : Invalid argument '%s' "
                        "(should be 'x0[%],y0[%],x1[%],y1[%][,color])",gmic_inds,argument);
      ++position; continue;
    }

    // Draw rectangle.
    if (!cimg::strcmp("-rectangle",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, arg3[256] = { 0 }, color[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, sepx1 = 0, sepy1 = 0, end = 0;
      float x0 = 0, y0 = 0, x1 = 0, y1 = 0;
      if (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%1023[0-9.eE,+-]",
                      arg0,arg1,arg2,arg3,color)>=4 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1) &&
          ((std::sscanf(arg2,"%f%c%c",&x1,&sepx1,&end)==2 && sepx1=='%') ||
           std::sscanf(arg2,"%f%c",&x1,&end)==1) &&
          ((std::sscanf(arg3,"%f%c%c",&y1,&sepy1,&end)==2 && sepy1=='%') ||
           std::sscanf(arg3,"%f%c",&y1,&end)==1)) {
        gmic_print("Draw rectangle (%g%s,%g%s) - (%g%s,%g%s) with color '%s' on image%s.",
                   x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",x1,sepx1=='%'?"%":"",y1,sepy1=='%'?"%":"",
                   color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0),
            nx1 = (int)round(sepx1=='%'?x1*img.width/101:x1),
            ny1 = (int)round(sepy1=='%'?y1*img.height/101:y1);
          img.draw_rectangle(nx0,ny0,nx1,ny1,col);
        }
      } else gmic_error("Draw rectangle on image%s : Invalid argument '%s' "
                        "(should be 'x0[%],y0[%],x1[%],y1[%][,color])",gmic_inds,argument);
      ++position; continue;
    }

    // Draw triangle.
    if (!cimg::strcmp("-triangle",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, arg3[256] = { 0 };
      char arg4[256] = { 0 }, arg5[256] = { 0 }, color[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, sepx1 = 0, sepy1 = 0, sepx2 = 0, sepy2 = 0, end = 0;
      float x0 = 0, y0 = 0, x1 = 0, y1 = 0, x2 = 0, y2 = 0;
      if (std::sscanf(argument,
                      "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                      "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                      "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c"
                      "%1023[0-9.eE,+-]",
                      arg0,arg1,arg2,arg3,arg4,arg5,color)>=6 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1) &&
          ((std::sscanf(arg2,"%f%c%c",&x1,&sepx1,&end)==2 && sepx1=='%') ||
           std::sscanf(arg2,"%f%c",&x1,&end)==1) &&
          ((std::sscanf(arg3,"%f%c%c",&y1,&sepy1,&end)==2 && sepy1=='%') ||
           std::sscanf(arg3,"%f%c",&y1,&end)==1) &&
          ((std::sscanf(arg4,"%f%c%c",&x2,&sepx2,&end)==2 && sepx2=='%') ||
           std::sscanf(arg4,"%f%c",&x2,&end)==1) &&
          ((std::sscanf(arg5,"%f%c%c",&y2,&sepy2,&end)==2 && sepy2=='%') ||
           std::sscanf(arg5,"%f%c",&y2,&end)==1)) {
        gmic_print("Draw triangle (%g%s,%g%s) - (%g%s,%g%s) - (%g%s,%g%s) with color '%s' on image%s.",
                   x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",
                   x1,sepx1=='%'?"%":"",y1,sepy1=='%'?"%":"",
                   x2,sepx2=='%'?"%":"",y2,sepy2=='%'?"%":"",
                   color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0),
            nx1 = (int)round(sepx1=='%'?x1*img.width/101:x1),
            ny1 = (int)round(sepy1=='%'?y1*img.height/101:y1),
            nx2 = (int)round(sepx2=='%'?x2*img.width/101:x2),
            ny2 = (int)round(sepy2=='%'?y2*img.height/101:y2);
          img.draw_triangle(nx0,ny0,nx1,ny1,nx2,ny2,col);
        }
      } else gmic_error("Draw triangle on image%s : Invalid argument '%s' "
                        "(should be 'x0[%],y0[%],x1[%],y1[%],x2[%],y2[%][,color])",gmic_inds,argument);
      ++position; continue;
    }

    // Draw ellipse.
    if (!cimg::strcmp("-ellipse",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, color[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, end = 0;
      float x0 = 0, y0 = 0, r0 = 0, r1 = 0, ru = 1, rv = 0;
      if (std::sscanf(argument,"%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%f%*c%f%*c%f%*c%f%*c%1023[0-9.eE,+-]",
                      arg0,arg1,&r0,&r1,&ru,&rv,color)>=4 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1)) {
        gmic_print("Draw ellipse centered at (%g%s,%g%s) with radii (%g,%g), orientation (%g,%g) and color '%s' on image%s.",
                   x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",
                   r0,r1,ru,rv,color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0);
          img.draw_ellipse(nx0,ny0,r0,r1,ru,rv,col);
        }
      } else gmic_error("Draw ellipse on image%s : Invalid argument '%s' "
                        "(should be 'x[%],y[%],r,R[,u,v[,color]])",gmic_inds,argument);
      ++position; continue;
    }

    // Draw text.
    if (!cimg::strcmp("-text",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, color[1024] = { 0 }, text[1024] = { 0 };
      char sepx0 = 0, sepy0 = 0, end = 0;
      float x0 = 0, y0 = 0; int siz = 11;
      if (std::sscanf(argument,"%1023[^,],%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%d%*c%1023[0-9.eE,+-]",
                      text,arg0,arg1,&siz,color)>=1 &&
          ((std::sscanf(arg0,"%f%c%c",&x0,&sepx0,&end)==2 && sepx0=='%') ||
           std::sscanf(arg0,"%f%c",&x0,&end)==1 || !arg0[0]) &&
          ((std::sscanf(arg1,"%f%c%c",&y0,&sepy0,&end)==2 && sepy0=='%') ||
           std::sscanf(arg1,"%f%c",&y0,&end)==1 || !arg1[0])) {
        gmic_print("Draw text \"%s\" at position (%g%s,%g%s) with font size %d and color '%s' on image%s.",
                   text,x0,sepx0=='%'?"%":"",y0,sepy0=='%'?"%":"",siz,color[0]?color:"default",gmic_inds);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx0 = (int)round(sepx0=='%'?x0*img.width/101:x0),
            ny0 = (int)round(sepy0=='%'?y0*img.height/101:y0);
          img.draw_text(nx0,ny0,text,col.ptr(),0,1,siz);
        }
      } else gmic_error("Draw text on image%s : Invalid argument '%s' "
                        "(should be 'text[,x[%],y[%][,size[,color]]]').",gmic_inds,argument);
      ++position; continue;
    }

    // Draw image.
    if (!cimg::strcmp("-image",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, sepx = 0, sepy = 0, sepz = 0, end = 0;
      int ind0 = 0, indm0 = -1; float x = 0, y = 0, z = 0;
      if (std::sscanf(argument,"[%d]%*c[%d]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]",
                      &ind0,&indm0,arg0,arg1,arg2)>=2 &&
          ((std::sscanf(arg0,"%f%c%c",&x,&sepx,&end)==2 && sepx=='%') ||
           std::sscanf(arg0,"%f%c",&x,&end)==1 || !arg0[0]) &&
          ((std::sscanf(arg1,"%f%c%c",&y,&sepy,&end)==2 && sepy=='%') ||
           std::sscanf(arg1,"%f%c",&y,&end)==1 || !arg1[0]) &&
          ((std::sscanf(arg2,"%f%c%c",&z,&sepz,&end)==2 && sepz=='%') ||
           std::sscanf(arg2,"%f%c",&z,&end)==1 || !arg2[0])) {
	gmic_check_indice(ind0,"Draw image%s");
	gmic_check_indice(indm0,"Draw image%s");
        gmic_print("Draw image [%d] at (%g%s,%g%s,%g%s) with opacity mask [%d] on image%s.",
                   ind0,x,sepx=='%'?"%":"",y,sepy=='%'?"%":"",z,sepz=='%'?"%":"",indm0,gmic_inds);
        const CImg<T> sprite = images[ind0], mask = images[indm0];
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            nx = (int)round(sepx=='%'?x*img.dimx()/101:x),
            ny = (int)round(sepy=='%'?y*img.dimy()/101:y),
            nz = (int)round(sepz=='%'?z*img.dimz()/101:z);
          img.draw_image(nx,ny,nz,sprite,mask);
        }
      } else if (std::sscanf(argument,"[%d]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]",
                             &ind0,arg0,arg1,arg2)>0 &&
                 ((std::sscanf(arg0,"%f%c%c",&x,&sepx,&end)==2 && sepx=='%') ||
                  std::sscanf(arg0,"%f%c",&x,&end)==1 || !arg0[0]) &&
                 ((std::sscanf(arg1,"%f%c%c",&y,&sepy,&end)==2 && sepy=='%') ||
                  std::sscanf(arg1,"%f%c",&y,&end)==1 || !arg1[0]) &&
                 ((std::sscanf(arg2,"%f%c%c",&z,&sepz,&end)==2 && sepz=='%') ||
                  std::sscanf(arg2,"%f%c",&z,&end)==1 || !arg2[0])) {
	gmic_check_indice(ind0,"Draw image%s");
        gmic_print("Draw image [%d] at (%g%s,%g%s,%g%s) on image%s.",
                   ind0,x,sepx=='%'?"%":"",y,sepy=='%'?"%":"",z,sepz=='%'?"%":"",gmic_inds);
        const CImg<T> sprite = images[ind0];
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          const int
            nx = (int)round(sepx=='%'?x*img.dimx()/101:x),
            ny = (int)round(sepy=='%'?y*img.dimy()/101:y),
            nz = (int)round(sepz=='%'?z*img.dimz()/101:z);
          img.draw_image(nx,ny,nz,sprite);
        }
      } else gmic_error("Draw image%s : Invalid argument '%s' "
                        "(should be '[ind_image],[ind_opacity],x,y[,z]' or "
                        "'[ind_image],x,y[,z]').",gmic_inds,argument);
      ++position; continue;
    }

    // Draw plasma fractal.
    if (!cimg::strcmp("-plasma",item0)) {
      float alpha = 1, beta = 1; char end = 0;
      if (std::sscanf(argument,"%f%*c%f%c",&alpha,&beta,&end)==2 ||
          std::sscanf(argument,"%f%c",&alpha,&end)==1) ++position;
      gmic_print("Draw plasma in image%s with alpha %g and beta %g.",gmic_inds,alpha,beta);
      cimg_foroff(indices,l) images[indices[l]].draw_plasma(alpha,beta);
      continue;
    }

    // Draw Mandelbrot/Julia fractal.
    if (!cimg::strcmp("-mandelbrot",item0)) {
      double z0r = -2, z0i = -2, z1r = 2, z1i = 2, paramr = 0, parami = 0;
      int itermax = 100, julia = 0;
      if (std::sscanf(argument,"%lf%*c%lf%*c%lf%*c%lf%*c%d%*c%d%*c%lf%*c%lf",
                      &z0r,&z0i,&z1r,&z1i,&itermax,&julia,&paramr,&parami)>0) ++position;
      gmic_print("Draw %s fractal in image%s from complex area (%g,%g)-(%g,%g) "
                 "with c0 = (%g,%g) (%d iterations).",
                 julia?"Julia":"Mandelbrot",gmic_inds,z0r,z0i,z1r,z1i,paramr,parami,itermax);
      cimg_foroff(indices,l)
        images[indices[l]].draw_mandelbrot(CImg<T>(),1,z0r,z0i,z1r,z1i,itermax,true,
                                           julia?true:false,paramr,parami);
      continue;
    }

    // Flood fill.
    if (!cimg::strcmp("-flood",item0)) {
      char arg0[256] = { 0 }, arg1[256] = { 0 }, arg2[256] = { 0 }, color[1024] = { 0 };
      char sepx = 0, sepy = 0, sepz = 0, end = 0;
      float x = 0, y = 0, z = 0; float tolerance = 0;
      if (std::sscanf(argument,
                      "%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%255[0-9.eE%+-]%*c%f%*c%1023[0-9.eE,+-]",
                      arg0,arg1,arg2,&tolerance,color)>=1 &&
          ((std::sscanf(arg0,"%f%c%c",&x,&sepx,&end)==2 && sepx=='%') ||
           std::sscanf(arg0,"%f%c",&x,&end)==1) &&
          ((std::sscanf(arg1,"%f%c%c",&y,&sepy,&end)==2 && sepy=='%') ||
           std::sscanf(arg1,"%f%c",&y,&end)==1 || !arg1[0]) &&
          ((std::sscanf(arg2,"%f%c%c",&z,&sepz,&end)==2 && sepz=='%') ||
           std::sscanf(arg2,"%f%c",&z,&end)==1 || !arg2[0])) {
        gmic_print("Flood fill image%s from (%g%s,%g%s,%g%s) with tolerance %g and color '%s'.",
                   gmic_inds,x,sepx=='%'?"%":"",y,sepy=='%'?"%":"",z,sepz=='%'?"%":"",tolerance,color);
        cimg_foroff(indices,l) {
          CImg<T> &img = images[indices[l]];
          CImg<T> col(img.dimv(),1,1,1,0);
          col.fill(color,true);
          const int
            nx = (int)round(sepx=='%'?x*img.dimx()/101:x),
            ny = (int)round(sepy=='%'?y*img.dimy()/101:y),
            nz = (int)round(sepz=='%'?z*img.dimz()/101:z);
          img.draw_fill(nx,ny,nz,col,1,tolerance);
        }
      } else gmic_error("Flood fill image%s : Invalid argument '%s' "
                        "(should be 'x[,y[,z[,tolerance[,color]]]]').",gmic_inds,argument);
      ++position; continue;
    }

    //-------------------------
    // Image list manipulation
    //-------------------------

    // Remove specified image(s).
    if (!cimg::strcmp("-remove",item0)) {
      gmic_print("Remove image%s",gmic_inds);
      unsigned int off = 0;
      cimg_foroff(indices,l) {
        const unsigned int ind = indices[l] - off;
        images.remove(ind); filenames.remove(ind);
        ++off;
      }
      if (!quiet) std::fprintf(stderr," (%u image%s left).",images.size,images.size==1?"":"s");
      continue;
    }

    // Keep specified image(s).
    if (!cimg::strcmp("-keep",item0) || !cimg::strcmp("-k",item0)) {
      gmic_print("Keep image%s",gmic_inds);
      CImgList<T> nimages(indices.size());
      cimg_foroff(indices,l) nimages[l].swap(images[indices[l]]);
      nimages.transfer_to(images);
      if (!quiet) std::fprintf(stderr," (%u image%s left).",images.size,images.size==1?"":"s");
      continue;
    }

    // Move image(s) to specified position.
    if (!cimg::strcmp("-move",item0)) {
      int ind0 = 0; char sep = 0, end = 0;
      if (std::sscanf(argument,"[%d%c%c",&ind0,&sep,&end)!=2 || sep!=']')
        gmic_error("Move image%s : Invalid argument '%s' "
                   "(should be '[indice]').",gmic_inds,argument);
      gmic_check_indice(ind0,"Move image%s");
      CImgList<T> nimages;
      CImgList<char> nfilenames;
      cimg_foroff(indices,l) {
        const unsigned int ind = indices[l];
        nimages.insert(1); nimages[nimages.size-1].swap(images[ind]);
        nfilenames.insert(1); nfilenames[nfilenames.size-1].swap(filenames[ind]);
      }
      images.insert(nimages,ind0); filenames.insert(nfilenames,ind0);
      cimglist_for(images,l) if (!images[l]) { images.remove(l); filenames.remove(l--); }
      ++position; continue;
    }

    // Reverse images order.
    if (!cimg::strcmp("-reverse",item0)) {
      gmic_print("Reverse images order.");
      CImgList<T> nimages(indices.size());
      CImgList<char> nfilenames(indices.size());
      cimg_foroff(indices,l) { nimages[l].swap(images[indices[l]]); nfilenames[l].swap(filenames[indices[l]]); }
      nimages.reverse(); nfilenames.reverse();
      cimg_foroff(indices,l) { nimages[l].swap(images[indices[l]]); nfilenames[l].swap(filenames[indices[l]]); }
      continue;
    }

    // Set image name.
    if (!cimg::strcmp("-name",item0)) {
      cimg_foroff(indices,l) filenames[indices[l]].assign(argument,cimg::strlen(argument)+1,1,1,1,false);
      ++position; continue;
    }

    //-------------------------
    // 3D objects manipulation
    //-------------------------

    // Compute 3D elevation.
    if (!cimg::strcmp("-elevation",item0)) {
      float zfact = 0.2f; char end = 0;
      if (std::sscanf(argument,"%f%c",&zfact,&end)==1) ++position;
      gmic_print("Build 3D elevation of image%s with z-factor %g.",gmic_inds,zfact);
      cimg_foroff(indices,l) {
        const unsigned int ind = indices[l];
        points3d[ind] = images[ind].get_elevation3d(primitives3d[ind],colors3d[ind],
                                                    images[ind].get_pointwise_norm(),zfact);
        if (ind>=4096) {
          gmic_warning("Build 3D elevation of image%s : Cannot create 3D object for indices %u>=4096.",
                       gmic_inds,ind);
          break;
        }
      }
      continue;
    }

    // Invert 3D orientation.
    if (!cimg::strcmp("-invert3d",item0)) {
      gmic_print("Invert orientation of 3D object%s.",gmic_inds);
      cimg_foroff(indices,l) primitives3d[indices[l]].invert_object3d();
      continue;
    }

    // Compute 3D isovalue.
    if (!cimg::strcmp("-isovalue",item0)) {
      float value = 0; char end = 0;
      if (std::sscanf(argument,"%f%c",&value,&end)==1) {
        gmic_print("Build 3D object for isovalue %g of image%s.",value,gmic_inds);
        cimg_foroff(indices,l) {
          const unsigned int ind = indices[l];
          const CImg<T>& img = images[ind];
          points3d[ind].assign();
          primitives3d[ind].assign();
          colors3d[ind].assign();
          CImg<unsigned char> palette;
          palette.assign(3,img.dim,1,1,220).noise(35,1);
          if (img.dim==1) palette(0) = palette(1) = palette(2) = 255;
          else {
            palette(0,0) = 255; palette(1,0) = 30; palette(2,0) = 30;
            palette(0,1) = 30; palette(1,1) = 255; palette(2,1) = 30;
            if (img.dim>=3) palette(0,2) = 30; palette(1,2) = 30; palette(2,2) = 255;
          }
          cimg_forV(img,k) {
            CImgList<unsigned int> prims;
            const CImg<float> pts = img.get_shared_channel(k).get_isovalue3d(prims,value);
            if (pts) {
              points3d[ind].append_object3d(primitives3d[ind],pts,prims);
              colors3d[ind].insert(prims.size,
                                   CImg<unsigned char>::vector(palette(0,k),palette(1,k),palette(2,k)));
            }
          }
          if (!points3d[ind])
            gmic_warning("Build 3D object for isovalue of image [%u] : Isovalue %g not found.",ind,value);
          if (ind>=4096) {
            gmic_warning("Build 3D object for isovalue of image%s : "
                         "Cannot create 3D object for indices %u>=4096.",gmic_inds,ind);
            break;
          }
        }
      } else gmic_error("Build 3D object for isovalue of image%s : Invalid argument '%s' "
                        "(should be 'isovalue').",gmic_inds,argument);
      ++position; continue;
    }

    //--------------------------
    // Input/Output and Display
    //--------------------------

    // Print.
    if (!cimg::strcmp("-print",item0) || !cimg::strcmp("-p",item0)) {
      if (images.size) {
        gmic_print("Print image%s.\n\n",gmic_inds);
        char title[1024];
        cimg_foroff(indices,l) {
          const unsigned int ind = indices[l];
          std::sprintf(title,"image [%u] = '%s'",ind,filenames[ind].ptr());
          images[ind].print(title);
        }
        released = true;
      } else gmic_print("Print image[].");
      continue;
    }

    // Display.
    if (!cimg::strcmp("-display",item0) || !cimg::strcmp("-d",item0)) {
      if (gmic_display(images,indices)) released = true;
      continue;
    }

    // Display 3D object.
    if (!cimg::strcmp("-display3d",item0) || !cimg::strcmp("-d3d",item0)) {
      if (gmic_display3d(indices)) released = true;
      continue;
    }

    // Display as a graph plot.
    if (!cimg::strcmp("-plot",item0)) {
      int plot_type = 1; double vmin = 0, vmax = 0;
      const int nb = std::sscanf(argument,"%d%*c%lf%*c%lf",&plot_type,&vmin,&vmax);
      if (!nb || nb==1 || nb==3) {
        if (nb) ++position;
        if (plot_type<0 || plot_type>3) gmic_error("Plot image%s : Invalid plot type %d.",gmic_inds,plot_type);
        released = gmic_display_plot(images,indices,plot_type,vmin,vmax);
      } else gmic_error("Plot image%s : Invalid argument '%s' "
                        "(should be 'plot_type[,value_min,value_max]').",gmic_inds,argument);
      continue;
    }

    // Output.
    if (!cimg::strcmp("-output",item0) || !cimg::strcmp("-o",item0)) {
      const char *const ext = cimg::split_filename(argument);
      if (!cimg::strcasecmp("off",ext)) {
        char filename[1024] = { 0 };
        std::strcpy(filename,argument);
        const unsigned int siz = indices.size();
        cimg_foroff(indices,l) {
          const unsigned int ind = indices[l];
          if (siz!=1) cimg::number_filename(argument,l,6,filename);
          gmic_print("Output 3D object [%u] as file '%s'.",ind,filename);
          points3d[ind].save_off(filename,primitives3d[ind],colors3d[ind]);
        }
      } else {
        CImgList<T> output_images;
        cimg_foroff(indices,l) output_images.insert(images[indices[l]],~0U,true);
        gmic_print("Output image%s as file '%s'.",gmic_inds,argument);
        output_images.save(argument);
      }
      released = true; ++position; continue;
    }

    // Exit.
    if (!cimg::strcmp("-quit",item0) || !cimg::strcmp("-q",item0)) {
      gmic_print("Quit.\n");
      std::exit(0);
    }

    // Input.
    if (!cimg::strcmp("-i",item0) || !cimg::strcmp("-input",item0)) ++position;
    else { argument = item; item1[0] = 0; }
    if (!cimg::strlen(item1)) indices.assign(1,1,1,1,images.size);
    CImgList<T> input_images;
    CImgList<char> input_filenames;
    bool obj3d = false;
    char st_inds[256] = { 0 }, stx[256] = { 0 }, sty[256] = { 0 }, stz[256] = { 0 }, stv[256] = { 0 };
    char end = 0, sep = 0, sepx = 0, sepy = 0, sepz = 0, sepv = 0;
    int nb = 1, indx = no_ind, indy = no_ind, indz = no_ind, indv = no_ind;
    float dx = 0, dy = 1, dz = 1, dv = 1;

    if (std::sscanf(argument,"[%255[0-9%,:-]]%*c%d%c",st_inds,&nb,&end)==2 ||
        (std::sscanf(argument,"[%255[0-9%,:-]%c%c",st_inds,&sep,&end)==2 && sep==']')) {

      // nb copies of existing sub-images.
      const CImg<unsigned int> indices0 = gmic_indices(st_inds,images.size,"-input");
      char st_tmp[256] = { 0 }; std::strcpy(st_tmp,gmic_string(indices0,true));
      if (nb<=0) gmic_error("Input %d copies of image%s : Invalid argument '%s'.",
                            nb,st_tmp,argument);
      if (nb!=1) gmic_print("Input %d copies of image%s at position%s",nb,st_tmp,gmic_inds);
      else gmic_print("Input copy of image%s at position%s",st_tmp,gmic_inds);
      for (int i = 0; i<nb; ++i) cimg_foroff(indices0,l) {
        input_images.insert(images[indices0[l]]);
        input_filenames.insert(filenames[indices0[l]]);
      }
    } else if (((std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%d%c",
                             stx,sty,stz,stv,&(nb=1),&end)==5 ||
                 std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",
                             stx,sty,stz,stv,&end)==4) &&
                (std::sscanf(stx,"%f%c",&dx,&end)==1 ||
                 (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%') ||
		 (std::sscanf(stx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']')) &&
                (std::sscanf(sty,"%f%c",&dy,&end)==1 ||
                 (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%') ||
		 (std::sscanf(sty,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']')) &&
                (std::sscanf(stz,"%f%c",&dz,&end)==1 ||
                 (std::sscanf(stz,"%f%c%c",&dz,&sepz,&end)==2 && sepz=='%') ||
		 (std::sscanf(stz,"[%d%c%c",&indz,&sepz,&end)==2 && sepz==']')) &&
                (std::sscanf(stv,"%f%c",&dv,&end)==1 ||
                 (std::sscanf(stv,"%f%c%c",&dv,&sepv,&end)==2 && sepv=='%') ||
		 (std::sscanf(stz,"[%d%c%c",&indv,&sepv,&end)==2 && sepv==']'))) ||
               (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",stx,sty,stz,&end)==3 &&
                (std::sscanf(stx,"%f%c",&dx,&end)==1 ||
                 (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%') ||
		 (std::sscanf(stx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']')) &&
                (std::sscanf(sty,"%f%c",&dy,&end)==1 ||
                 (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%') ||
		 (std::sscanf(sty,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']')) &&
                (std::sscanf(stz,"%f%c",&dz,&end)==1 ||
                 (std::sscanf(stz,"%f%c%c",&dz,&sepz,&end)==2 && sepz=='%') ||
		 (std::sscanf(stz,"[%d%c%c",&indz,&sepz,&end)==2 && sepz==']'))) ||
               (std::sscanf(argument,"%255[][0-9.eE%+-]%*c%255[][0-9.eE%+-]%c",stx,sty,&end)==2 &&
                (std::sscanf(stx,"%f%c",&dx,&end)==1 ||
                 (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%') ||
		 (std::sscanf(stx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']')) &&
                (std::sscanf(sty,"%f%c",&dy,&end)==1 ||
                 (std::sscanf(sty,"%f%c%c",&dy,&sepy,&end)==2 && sepy=='%') ||
		 (std::sscanf(sty,"[%d%c%c",&indy,&sepy,&end)==2 && sepy==']'))) ||
               (std::sscanf(argument,"%255[][0-9.eE%+-]%c",stx,&end)==1 &&
                (std::sscanf(stx,"%f%c",&dx,&end)==1 ||
                 (std::sscanf(stx,"%f%c%c",&dx,&sepx,&end)==2 && sepx=='%') ||
		 (std::sscanf(stx,"[%d%c%c",&indx,&sepx,&end)==2 && sepx==']')))) {

      // nb copies of a new black image.
      if (indx!=no_ind) { gmic_check_indice(indx,"Input black image%s"); dx = (float)images[indx].dimx(); sepx = 0; }
      if (indy!=no_ind) { gmic_check_indice(indy,"Input black image%s"); dy = (float)images[indy].dimy(); sepy = 0; }
      if (indz!=no_ind) { gmic_check_indice(indz,"Input black image%s"); dz = (float)images[indz].dimz(); sepz = 0; }
      if (indv!=no_ind) { gmic_check_indice(indv,"Input black image%s"); dv = (float)images[indv].dimv(); sepv = 0; }
      if (sepx=='%') { dx = images.size?dx*images[images.size-1].dimx()/100:0; if (!(int)dx) ++dx; }
      if (sepy=='%') { dy = images.size?dy*images[images.size-1].dimy()/100:0; if (!(int)dy) ++dy; }
      if (sepz=='%') { dz = images.size?dz*images[images.size-1].dimz()/100:0; if (!(int)dz) ++dz; }
      if (sepv=='%') { dv = images.size?dv*images[images.size-1].dimv()/100:0; if (!(int)dv) ++dv; }

      if (nb<=0) gmic_error("Input %d black image%s : Invalid number of copies.",nb,gmic_inds);
      if (dx<=0 || dy<=0 || dz<=0 || dv<=0)
        gmic_error("Input %d black image%s : Invalid image dimensions %gx%gx%gx%g.",
                   nb,gmic_inds,dx,dy,dz,dv);
      if (nb!=1) gmic_print("Input %d black images at position%s",nb,gmic_inds);
      else gmic_print("Input black image at position%s",gmic_inds);
      CImg<T> empty((int)dx,(int)dy,(int)dz,(int)dv,0);
      input_images.insert(nb-1,empty); input_images.insert(1);
      input_images[input_images.size-1].swap(empty);
      filenames.insert(input_images.size,CImg<char>("(gmic)",7,1,1,1,false));
    } else {

      // Insert a loaded filename.
      char filename[1024] = { 0 }, options[1024] = { 0 }; const char *ext = 0, *basename = 0;
      if (argument[0]!='-' || (argument[1] && argument[1]!='.')) {
        std::FILE *file = std::fopen(argument,"r");
        if (file) { std::fclose(file); std::strcpy(filename,argument); }
        else {
          std::sscanf(argument,"%1023[^:]:%s",filename,options);
          if (!(file=std::fopen(filename,"r")))
            gmic_error("Input file '%s' : File not found.",filename);
          std::fclose(file);
        }
      } else std::strcpy(filename,argument);
      basename = cimg::basename(filename);
      ext = cimg::split_filename(filename);

      if (!cimg::strcasecmp("off",ext)) {  // 3D object file.
        if (indices.size()!=1 || indices[indices.size()-1]!=images.size)
          gmic_warning("Input 3D object '%s' : Indice%s will be ignored.",filename,gmic_inds);
        gmic_print("Input 3D object '%s'",filename);
        input_images.insert(1);
        input_filenames.insert(CImg<char>(fullnames?filename:basename,
                                          cimg::strlen(fullnames?filename:basename)+1,1,1,1,false));
        indices.assign(1,1,1,1,images.size);
        points3d[images.size].load_off(filename,primitives3d[images.size],colors3d[images.size]);
        obj3d = true;
      } else if (!cimg::strcasecmp("mpg",ext) || // Movie file.
                 !cimg::strcasecmp("mpeg",ext) ||
                 !cimg::strcasecmp("mp2",ext) ||
                 !cimg::strcasecmp("mp4",ext) ||
                 !cimg::strcasecmp("avi",ext) ||
                 !cimg::strcasecmp("divx",ext) ||
                 !cimg::strcasecmp("asx",ext) ||
                 !cimg::strcasecmp("ogg",ext) ||
                 !cimg::strcasecmp("ogm",ext) ||
                 !cimg::strcasecmp("flv",ext) ||
                 !cimg::strcasecmp("rm",ext) ||
                 !cimg::strcasecmp("wmv",ext) ||
                 !cimg::strcasecmp("mov",ext) ||
                 !cimg::strcasecmp("qt",ext)) {
	unsigned int first = 0, last = ~0U, step = 1;
	const int nb = std::sscanf(options,"%u%*c%u%*c%u",&first,&last,&step);
	switch (nb) {
	case 3: gmic_print("Input frames %u...%u:%u of file '%s'",first,last,step,filename); break;
	case 2: gmic_print("Input frames %u...%u of file '%s'",first,last,filename); break;
	case 1: gmic_print("Input frames %u..(last) of file '%s'",first,filename); break;
	default: gmic_print("Input file '%s'",filename); break;
	}
	input_images.load_ffmpeg(filename,first,last,step);
        input_filenames.insert(input_images.size,CImg<char>(fullnames?filename:basename,
                                                            cimg::strlen(fullnames?filename:basename)+1,1,1,1,false));
      } else if (!cimg::strcasecmp("raw",ext)) { // Raw file.
        int dx = 0, dy = 1, dz = 1, dv = 1;
	if (std::sscanf(options,"%d%*c%d%*c%d%*c%d",&dx,&dy,&dz,&dv)>0) {
          if (dx<=0 || dy<=0 || dz<=0 || dv<=0)
            gmic_error("Input raw file '%s' : Invalid specified dimensions %dx%dx%dx%d.",filename,dx,dy,dz,dv);
	  gmic_print("Input raw file '%s'",filename);
	  input_images.insert(1); input_images[0].load_raw(filename,dx,dy,dz,dv);
	  input_filenames.insert(CImg<char>(fullnames?filename:basename,
                                            cimg::strlen(fullnames?filename:basename)+1,1,1,1,false));
	} else gmic_error("Input raw file '%s' : Image dimensions must be specified.",filename);
      } else if (!cimg::strcasecmp("yuv",ext)) { // YUV file.
        int dx = 0, dy = 0; unsigned int first = 0, last = ~0U, step = 1;
        if (std::sscanf(options,"%d%*c%d%*c%u%*c%u%*c%u",&dx,&dy,&first,&last,&step)>0) {
          if (dx<=0 || dy<=0)
            gmic_error("Input yuv file '%s' : Invalid specified dimensions %dx%d.",filename,dx,dy);
          gmic_print("Input yuv file '%s'",filename);
          input_images.load_yuv(filename,dx,dy,first,last,step);
          input_filenames.insert(input_images.size,CImg<char>(fullnames?filename:basename,
                                                              cimg::strlen(fullnames?filename:basename)+1,1,1,1,false));
        } else gmic_error("Input yuv file '%s' : Image dimensions must be specified.",filename);
      } else { // Other file type.
        gmic_print("Input file '%s'",filename);
        input_images.load(filename);
        input_filenames.insert(input_images.size,
                               CImg<char>(fullnames?filename:basename,
                                          cimg::strlen(fullnames?filename:basename)+1,1,1,1,false));
      }
    }

    if (!quiet) {
      const unsigned int last = input_images.size-1;
      if (obj3d)
        std::fprintf(stderr," (%d points, %u primitives).",
                     points3d[images.size-1].dimx(),primitives3d[images.size-1].size);
      else if (input_images.size==1)
        std::fprintf(stderr," (1 image %ux%ux%ux%u).",
                     input_images[0].width,input_images[0].height,input_images[0].depth,
                     input_images[0].dim);
      else std::fprintf(stderr," (%u images [0] = %ux%ux%ux%u, ..., [%u] = %ux%ux%ux%u).",
                        input_images.size,
                        input_images[0].width,input_images[0].height,input_images[0].depth,
                        input_images[0].dim,last,
                        input_images[last].width,input_images[last].height,input_images[last].depth,
                        input_images[last].dim);
    }

    for (unsigned int l = 0, siz = indices.size()-1, off = 0; l<=siz; ++l) {
      const unsigned int ind = indices[l] + off;
      if (l!=siz) images.insert(input_images,ind);
      else {
        images.insert(input_images.size,ind);
        cimglist_for(input_images,k) images[ind+k].swap(input_images[k]);
      }
      filenames.insert(input_filenames,ind);
      off+=input_images.size;
    }

  }

  // Display final result if necessary (not 'released' before).
  if ((int)position==argc && !released) {
    if (!gmic_display3d(CImg<unsigned int>::sequence(images.size,0,images.size-1)))
      gmic_display(images,CImg<unsigned int>::sequence(images.size,0,images.size-1));
  }

}

#ifndef gmic_debug
#if defined(gmic_bool) || !defined(gmic_separate_compilation)
void gmic_parse_bool(CImgList<bool> &images) { gmic_parse(images); }
#endif
#if defined(gmic_uchar) || !defined(gmic_separate_compilation)
void gmic_parse_uchar(CImgList<unsigned char> &images) { gmic_parse(images); }
#endif
#if defined(gmic_char) || !defined(gmic_separate_compilation)
void gmic_parse_char(CImgList<char> &images) { gmic_parse(images); }
#endif
#if defined(gmic_ushort) || !defined(gmic_separate_compilation)
void gmic_parse_ushort(CImgList<unsigned short> &images) { gmic_parse(images); }
#endif
#if defined(gmic_short) || !defined(gmic_separate_compilation)
void gmic_parse_short(CImgList<short> &images) { gmic_parse(images); }
#endif
#if defined(gmic_uint) || !defined(gmic_separate_compilation)
void gmic_parse_uint(CImgList<unsigned int> &images) { gmic_parse(images); }
#endif
#if defined(gmic_int) || !defined(gmic_separate_compilation)
void gmic_parse_int(CImgList<int> &images) { gmic_parse(images); }
#endif
#if defined(gmic_double) || !defined(gmic_separate_compilation)
void gmic_parse_double(CImgList<double> &images) { gmic_parse(images); }
#endif
#endif
#if defined(gmic_float) || !defined(gmic_separate_compilation)
void gmic_parse_float(CImgList<float> &images) { gmic_parse(images); }
#endif

// Main procedure.
//-----------------
#if defined(gmic_main) || !defined(gmic_separate_compilation)
#include "gmic.h"
int main(int argc, char **argv) {
  static const bool quiet = cimg_option("-quiet",false,0);
  gmic_globals::argc = argc;
  gmic_globals::argv = argv;
  if (argc==1) {
    gmic_print("No options or data provided. Try '%s -h' for help.\n",cimg::basename(argv[0]));
    std::exit(0);
  }

  // Load and define macro commands.
  CImgList<char> macros, commands;
  char mac[1024] = { 0 }, com[256*1024] = { 0 }, line[256*1024] = { 0 }; int err = 0;

  const char *filename = cimg_option("-macros",(char*)0,0);
  if (!filename) filename = cimg_option("-m",(char*)0,0);
  if (filename) {
    std::FILE *file = std::fopen(filename,"r");
    if (!file) gmic_warning("Load macro file '%s' : File not found.",filename);
    else {
      gmic_print("Load macro file '%s'",filename);
      while ((err=std::fscanf(file,"%262143[^\n]\n",line)>=0)) {
        if (err && line[0]!='#' && std::sscanf(line,"%1023s : %262143[^\n]\n",mac,com)==2) {
          macros.insert(CImg<char>(mac,cimg::strlen(mac)+1,1,1,1,false));
          commands.insert(CImg<char>(com,cimg::strlen(com),1,1,1,false));
        }
      }
      std::fclose(file);
      unsigned int nb_macros = macros.size;
      cimglist_for(macros,l) if (macros(l,0)=='_') --nb_macros;
      if (!quiet) {
        if (nb_macros) std::fprintf(stderr," (define %d new command%s).",nb_macros,nb_macros>1?"s":"");
        else std::fprintf(stderr,".");
      }
    }
  }

  const char *data = data_gmic, *const data_end = data_gmic + sizeof(data_gmic);
  while (data<data_end) {
    if (*data=='\n') ++data;
    else {
      if (std::sscanf(data,"%262143[^\n]\n",line)>0) data+=cimg::strlen(line)+1;
      if (line[0]!='#' && std::sscanf(line,"%1023s : %262143[^\n]\n",mac,com)==2) {
        macros.insert(CImg<char>(mac,cimg::strlen(mac)+1,1,1,1,false));
        commands.insert(CImg<char>(com,cimg::strlen(com),1,1,1,false));
      }
    }
  }

  // Display help.
  if (cimg_option("-h",false,0) || cimg_option("-help",false,0) || cimg_option("--help",false,0)) {
#ifndef _MSC_VER
    stderr = stdout;
#endif
    cimg_usage("GREYC's Magic Image Converter");
    cimg_help("        Version 0.8, Copyright (C) 2008, David Tschumperle (http://gmic.sourceforge.net)");

    cimg_help("\n  Usage\n"
              "  -----");
    cimg_help("  gmic [file_1] [file_2] .. [file_n] [command1] .. [command_n] [file_n+1] ...\n");
    cimg_help("  G'MIC is a console-based image processing tool whose goal is to convert, manipulate and");
    cimg_help("  visualize generic 1D/2D/3D multi-spectral image files. It is based on these simple rules :");
    cimg_help("    - G'MIC deals with a numbered list of images. All images are stored in memory.");
    cimg_help("    - First image of the list has indice 0.");
    cimg_help("    - Command line items orders how to add/remove/process images in the list.");
    cimg_help("    - Command line items are executed in the order they appear on the command line.");
    cimg_help("    - An item starting by '-' is either a global option or a command.");
    cimg_help("    - A global option is read only once. Multiple instances are then useless.");
    cimg_help("    - On the contrary, a command may appear more than one time, at different locations.");
    cimg_help("    - A command or a global option may have one mandatory or optional argument ");
    cimg_help("      (but no more than one).");
    cimg_help("    - Other command line items are considered either as input filenames or special input strings.");
    cimg_help("    - If one item is a filename, the corresponding image data are loaded and added to the list.");
    cimg_help("    - Special input strings can be used to insert new images to the list. They can be :");
    cimg_help("       'width[%][xheight[%][xdepth[%][xdim[%][xN]]]]' : Insert N black images with specified size.");
    cimg_help("       (in this case, adding '%' to a dimension means 'dimension percentage of the last image'),");
    cimg_help("       '[indice]' or '[indice]xN' : Insert 1 or N copies of image [indice].");
    cimg_help("    - When loading data, file formats are recognized by their filename extension.");
    cimg_help("    - Special filenames are '-' or '-.ext', meaning 'standard input/output' with");
    cimg_help("      default or specified 'ext' format.");
    cimg_help("    - Most of the commands may be restricted to specific images of the list, by adding '[]' to");
    cimg_help("      the command name. Several expressions are possible, for instance : ");
    cimg_help("       '[0,1,3]' : Restrict to images 0,1 and 3.");
    cimg_help("       '[3-5]' : Restrict to images 3 to 5.");
    cimg_help("       '[50%-100%] : Restrict to the second half of the image list.");
    cimg_help("       '[0,-1,-2]' : Restrict to the first and two latest images.");
    cimg_help("       '[0-9:3]' : Restrict to images 0 to 9, with a step of 3, i.e. images 0,3,6,9.");
    cimg_help("       '[0,2-4,50%--1]' : Restrict to images 0,2,3,4 and the second half of the list.");
    cimg_help("   - Some of the command names have a shortened equivalence.");
    cimg_help("   - Some input filenames or commands may result to the generation of 3D objects.");
    cimg_help("   - A simple 3D viewer is included in G'MIC to display 3D objects.");
    cimg_help("   - A 3D object created from an image has the same indice as the generating image.");
    cimg_help("   - Custom user-defined G'MIC commands can be defined with the use of a macro file.");
    cimg_help("   - A macro file is a simple text file, each line being of the form");
    cimg_help("     'command_name : substitution', or '# comment'.");
    cimg_help("   - A macro name starting by '_' is hidden and does not appear in the displayed macro list.");
    cimg_help("   - When invoking G'MIC, each macro command is expanded as its defined content.");
    cimg_help("   - Only one macro file can be specified at a time (using the global option '-macros' or '-m').");
    cimg_help("   - An example of a default macro file 'gmic_def.raw' is distributed within the G'MIC package.");
    cimg_help("   - Explicit inclusion of the default macro file is useless, since corresponding commands are already");
    cimg_help("     included by default in G'MIC.");

    cimg_help("\n  A list of possible commands and global options is available below.");
    cimg_help("  A parameter specified in '[]' is optional, except when standing for '[indices]' where it");
    cimg_help("  corresponds to one or several image indices as described above. In this case, the '[' and ']'");
    cimg_help("  characters must explicitely appear in the corresponding argument.");

    cimg_help("\n  Global options\n"
              "  --------------");
    cimg_option("-h",false,"Display this help (eq. to '-help' and '--help').");
    cimg_option("-quiet",false,"Do not show status messages.");
    cimg_option("-macros","'filename'","Load macro file from specified filename (eq. to '-m').");
    cimg_option("-fullnames",false,"Display full filenames in messages and window titles.");
    cimg_option("-back3d","'R,G,B'","Define background color in 3D viewer.");
    cimg_option("-r3d",4,"Set static rendering mode in 3D viewer ");
    cimg_help("                                              "
              "(-1=bounding box, 0=points, 1=lines, 2=faces, 3=flat, 4=gouraud, 5=phong).");
    cimg_option("-rd3d",-1,"Set dynamic rendering mode in 3D viewer (same as above).");
    cimg_option("-orient3d",false,"Consider oriented 3D objects in 3D viewer.");
    cimg_option("-specl3d",0.15f,"Amount of specular light in 3D viewer.");
    cimg_option("-specs3d",0.8f,"Shininess of specular light in 3D viewer.");

    cimg_help("\n  Commands : Arithmetic operators\n"
              "  -------------------------------");
    cimg_option("-add","'value', '[indice]' or (no args)","Add 'value' or '[indice]' to image(s)");
    cimg_help("                                              "
              "or add image(s) together (eq. to '-+').");
    cimg_option("-sub","'value', '[indice]' or (no args)","Substract 'value' or '[indice]' to image(s)");
    cimg_help("                                              "
              "or substract image(s) together (eq. to '--').");
    cimg_option("-mul","'value', '[indice]' or (no args)","Multiply image(s) by 'value' or '[indice]'");
    cimg_help("                                              "
              "or multiply image(s) together (eq. to '-*').");
    cimg_option("-div","'value', '[indice]' or (no args)","Divide image(s) by 'value' or '[indice]'");
    cimg_help("                                              "
              "or divide image(s) together (eq. to '-/').");
    cimg_option("-pow","'value', '[indice]' or (no args)","Compute image(s) to the power of 'value' or '[indice]'");
    cimg_help("                                              "
              "or power of the image(s) together (eq. to '-^').");
    cimg_option("-min","'value', '[indice]' or (no args)","Compute minimum between image(s) and 'value' or '[indice]'");
    cimg_help("                                              "
              "or minimum of image(s) together.");
    cimg_option("-max","'value', '[indice]' or (no args)","Compute maximum between image(s) and 'value' or '[indice]'");
    cimg_help("                                              "
              "or maximum of image(s) together.");
    cimg_option("-mod","'value', '[indice]' or (no args)","Compute modulo of image(s) with 'value' or '[indice]'");
    cimg_help("                                              "
              "or modulo with image(s) together.");
    cimg_option("-and","'value', '[indice]' or (no args)","Compute bitwise AND of image(s) with 'value' or '[indice]'");
    cimg_help("                                              "
              "or bitwise AND of image(s) together.");
    cimg_option("-or","'value', '[indice]' or (no args)","Compute bitwise OR of image(s) with 'value' or '[indice]'");
    cimg_help("                                              "
              "or bitwise OR of image(s) together.");
    cimg_option("-xor","'value', '[indice]' or (no args)","Compute bitwise XOR of image(s) with 'value' '[indice]'");
    cimg_help("                                              "
              "or bitwise XOR of image(s) together.");
    cimg_option("-cos","(no args)","Compute cosine of image(s) values.");
    cimg_option("-sin","(no args)","Compute sine of image(s) values.");
    cimg_option("-tan","(no args)","Compute tangent of image(s) values.");
    cimg_option("-acos","(no args)","Compute arccosine of image(s) values.");
    cimg_option("-asin","(no args)","Compute arcsine of image(s) values.");
    cimg_option("-atan","(no args)","Compute arctangent of image(s) values.");
    cimg_option("-abs","(no args)","Compute absolute value of image(s) values.");
    cimg_option("-sqr","(no args)","Compute square of image(s) values.");
    cimg_option("-sqrt","(no args)","Compute square root of image(s) values.");
    cimg_option("-exp","(no args)","Compute exponential of image(s) values.");
    cimg_option("-log","(no args)","Compute logarithm of image(s) values.");
    cimg_option("-log10","(no args)","Compute logarithm_10 of image(s) values.");

    cimg_help("\n  Commands : Pointwise pixel manipulation\n"
              "  ---------------------------------------");
    cimg_option("-type","'value_type'","Cast all images into specified value type");
    cimg_help("                                              "
              "('value_type' can be 'bool','uchar','char','ushort','short',");
    cimg_help("                                              "
              "'uint','int','float','double').");
    cimg_option("-endian","(no args)","Invert endianness of the image buffers.");
    cimg_option("-fill","'value1,value2,...' or '[indice]'","Fill image(s) with specified values in a repetitive way.");
    cimg_option("-threshold","'value[%]' or (noargs)","Threshold pixel values ((noargs) runs interactive mode).");
    cimg_option("-cut","'{value1[%],[indice]},{value2[%],[indice]}' or (noargs)","Cut pixel values in specified range ");
    cimg_help("                                              "
              "((noargs) runs interactive mode).");
    cimg_option("-normalize","'{value1[%],[indice]},{value2[%],[indice]}' or (noargs)",
                "Normalize pixel values in specified range (eq. to '-n').");
    cimg_option("-equalize","(noargs) or 'nb_levels'","Equalize histogram of image(s) using 'nb_levels' levels.");
    cimg_option("-quantize","'nb_levels'","Quantize image(s) using 'nb_levels' levels.");
    cimg_option("-noise","'variance[%][,noise_type]'","Add noise with specified variance and type");
    cimg_help("                                              "
              "('noise_type' can be '{0=gaussian, 1=uniform, 2=salt&pepper, 3=poisson}'.");
    cimg_option("-norm","(no args)","Compute pointwise L2-norm of multispectral image(s).");
    cimg_option("-orientation","(no args)","Compute pointwise orientation of multispectral image(s).");

    cimg_help("\n  Commands : Color bases conversions\n"
              "  ----------------------------------");
    cimg_option("-rgb2hsv","(no args)","Convert image(s) from RGB to HSV colorbases.");
    cimg_option("-rgb2hsl","(no args)","Convert image(s) from RGB to HSL colorbases.");
    cimg_option("-rgb2hsi","(no args)","Convert image(s) from RGB to HSI colorbases.");
    cimg_option("-rgb2yuv","(no args)","Convert image(s) from RGB to YUV colorbases.");
    cimg_option("-rgb2ycbcr","(no args)","Convert image(s) from RGB to YCbCr colorbases.");
    cimg_option("-rgb2xyz","(no args)","Convert image(s) from RGB to XYZ colorbases.");
    cimg_option("-rgb2lab","(no args)","Convert image(s) from RGB to Lab colorbases.");
    cimg_option("-rgb2cmy","(no args)","Convert image(s) from RGB to CMY colorbases.");
    cimg_option("-rgb2cmyk","(no args)","Convert image(s) from RGB to CMYK colorbases.");
    cimg_option("-rgb2lut","'[indice][,dithering]' or 'LUT_type[,dithering]'","Index image(s) with color LUT ");
    cimg_help("                                              "
              "('LUT_type' can be '{0=default, 1=rainbow, 2=contrast}'");
    cimg_help("                                              "
              "'dithering' can be '{0,1}').");
    cimg_option("-hsv2rgb","(no args)","Convert image(s) from HSV to RGB colorbases.");
    cimg_option("-hsl2rgb","(no args)","Convert image(s) from HSL to RGB colorbases.");
    cimg_option("-hsi2rgb","(no args)","Convert image(s) from HSI to RGB colorbases.");
    cimg_option("-yuv2rgb","(no args)","Convert image(s) from YUV to RGB colorbases.");
    cimg_option("-ycbcr2rgb","(no args)","Convert image(s) from YCbCr to RGB colorbases.");
    cimg_option("-xyz2rgb","(no args)","Convert image(s) from XYZ to RGB colorbases.");
    cimg_option("-lab2rgb","(no args)","Convert image(s) from Lab to RGB colorbases.");
    cimg_option("-cmy2rgb","(no args)","Convert image(s) from CMY to RGB colorbases.");
    cimg_option("-cmyk2rgb","(no args)","Convert image(s) from CMYK to RGB colorbases.");
    cimg_option("-lut2rgb","'[indice]' or 'LUT_type'","Map indexed LUT to image(s) ");
    cimg_help("                                              "
              "('LUT_type' can be '{0=default, 1=rainbow, 2=contrast}'.");

    cimg_help("\n  Commands : Geometric manipulation\n"
                "  ---------------------------------");
    cimg_option("-resize","'[indice][,interpolation[,borders[,center]]]' or ","");
    cimg_help("                     "
	      "'{[indice],width[%]}[x{[indice],height[%]}[x{[indice],depth[%]}[x{[indice],dim[%]}[,interpolation[,borders[,center]]]]]]'");
    cimg_help("                                              "
              "Resize image(s) to specified geometry (eq. to '-r')");
    cimg_help("                                              "
              "('interpolation' can be '{0=none, 1=nearest neighbor, 2=moving average,");
    cimg_help("                                              "
              "3=linear, 4=grid, 5=bicubic}')");
    cimg_option("-resize2x","(no args)","Resize image(s) using the Scale2x algorithm.");
    cimg_option("-resize3x","(no args)","Resize image(s) using the Scale3x algorithm.");
    cimg_option("-crop","(no args) or 'x0[%],x1[%][,border_conditions]' or 'x0[%],y0[%],x1[%],y1[%][,border_conditions]' or ","");
    cimg_help("                                              "
              "'x0[%],y0[%],z0[%],x1[%],y1[%],z1[%][,border_conditions]' or ");
    cimg_help("                                              "
              "'x0[%],y0[%],z0[%],v0[%],x1[%],y1[%],z1[%],v1[%][,border_conditions]'");
    cimg_help("                                              "
              "Crop image(s) using specified geometry (eq. to '-c') ");
    cimg_help("                                              "
              "('border_conditions' can be '{0=black, 1=nearest}')");
    cimg_help("                                              "
              "((no args) runs interactive mode).");
    cimg_option("-channels","'{[ind0],v0[%]}[,{[ind1],v1[%]}]'","Select channels v0..v1 of multispectral image(s).");
    cimg_option("-slices","'{[ind0],z0[%]}[,{[ind1],z1[%]}]'","Select slices z0..z1 of volumetric image(s).");
    cimg_option("-rotate","'angle[,border_conditions]'","Rotate image(s) of a given angle ");
    cimg_help("                                              "
              "('border_conditions' can be '{0=black, 1=nearest, 2=repeat, 3=linear}').");
    cimg_option("-mirror","'axis'",
                "Mirror image(s) along specified axis ('axis' can be '{x,y,z,v}').");
    cimg_option("-translate","'tx[%][,ty[%][,tz[%][,tv[%][,border_conditions]]]]'",
                "Translate image(s) by vector (dx,dy,dz,dv)");
    cimg_help("                                              "
              "('border_conditions' can be '{0=black, 1=nearest, 2=repeat}').");
    cimg_option("-transpose","(no args)","Transpose image(s).");
    cimg_option("-permute","'permutation'","Permute image axes by the specified permutation "
                "('permutation' can be 'yxzv',...).");
    cimg_option("-unroll","'axis'",
                "Unroll image(s) along specified axis ('axis' can be '{x,y,z,v}').");
    cimg_option("-split","'axis[,nb_parts]' or 'value[,keep]'",
                "Split image(s) along specified axis or value ('axis' can be '{x,y,z,v}').");
    cimg_option("-append","'axis,[alignement]'","Append image(s) along specified axis and alignement"
                "(axis can be '{x,y,z,v}' and ");
    cimg_help("                                              "
              "'alignement' can be '{p=left, c=center, n=right)'.");
    cimg_option("-warp","'[indice][,nb_frames[,interpolation[,border_conditions]]]'",
                "Warp image(s) to 'nb_frames' with field '[indice]' ");
    cimg_help("                                              "
              "('interpolation' can be '{0,1}', "
              "'border_conditions' can be '{0=black, 1=nearest}').");

    cimg_help("\n  Commands : Image filtering\n"
              "  --------------------------");
    cimg_option("-blur","'variance[,border_conditions]'",
                "Apply gaussian blur of specified variance");
    cimg_help("                                              "
              "('border_conditions' can be '{0=black, 1=nearest}').");
    cimg_option("-bilateral","'stdevs,stdevr'",
                "Apply bilateral filtering with specified standard deviations 'stdevs' and 'stdevr'.");
    cimg_option("-smooth","'amplitude[,sharpness[,anisotropy[,alpha[,sigma]]]]'",
                "Smooth image(s) anisotropically with specified parameters.");
    cimg_option("-denoise","'patch_size[,sigma_p[,sigma_s[,lookup_size]]]'",
                "Denoise image(s) with a patch-averaging procedure.");
    cimg_option("-median","'[size]'","Apply median filter of specified size.");
    cimg_option("-sharpen","'amplitude[,0]' or 'amplitude,1[,edge[,alpha[,sigma]]]'",
                "Sharpen image(s) with inverse diffusion or shock filters.");
    cimg_option("-convolve","'[indice][,border_conditions]'",
                "Convolve image(s) by the specified mask");
    cimg_help("                                              "
              "('border_conditions' can be '{0=black, 1=nearest}').");
    cimg_option("-correlate","'[indice][,border_conditions]'",
                "Correlate image(s) by the specified mask (same parameters as above)').");
    cimg_option("-erode","'size[,border_conditions]' or '[indice][,border_conditions]'","");
    cimg_help("                                              "
              "Erode image(s) by the specified mask (same parameters as above)').");
    cimg_option("-dilate","'size[,border_conditions]' or '[indice][,border_conditions]'","");
    cimg_help("                                              "
              "Dilate image(s) by the specified mask (same parameters as above)').");
    cimg_option("-gradient","'x', 'xy', 'xyz' or (no args)","Compute image gradient.");
    cimg_option("-hessian","(no args)","Compute image Hessian.");
    cimg_option("-fft","(no args)","Compute direct Fourier transform.");
    cimg_option("-ifft","(no args)","Compute inverse Fourier transform.");

    cimg_help("\n  Commands : Image creation and drawing\n"
              "  -------------------------------------");
    cimg_option("-dimensions","(no args)","Get dimensions of the image(s) as a 1x4 vector.");
    cimg_option("-stats","(no args)","Get statistics of the image(s) as a 1x6 vector.");
    cimg_option("-histogram","'[nb_values]'","Compute histogram of image(s) with 'nb_values' values.");
    cimg_option("-distance","'isovalue'","Compute distance function(s) to specified isovalue.");
    cimg_option("-label","(no args)","Label connected components of image(s).");
    cimg_option("-displacement","[indice][,smoothness]","Estimate smooth displacement field between image(s) "
                "and specified target '[indice]'.");
    cimg_option("-sort","(no args)","Sort values of image(s) in increasing order.");
    cimg_option("-psnr","'[max_intensity]'","Compute PSNR between specified image(s).");
    cimg_option("-point","'x[%],y[%][,z[%][,color]]'","Draw 3D colored point on specified image(s).");
    cimg_option("-line","'x0[%],y0[%],x1[%],y1[%][,color]'","Draw 2D colored line on specified image(s).");
    cimg_option("-rectangle","'x0[%],y0[%],x1[%],y1[%][,color]'","Draw 2D colored rectangle on specified image(s).");
    cimg_option("-triangle","'x0[%],y0[%],x1[%],y1[%],x2[%],y2[%][,color]'","Draw 2D colored triangle on specified image(s).");
    cimg_option("-ellipse","'x[%],y[%],r,R[,u,v[,color]]'","Draw 2D colored ellipse on specified image(s).");
    cimg_option("-text","text,x[%],y[%],size[,color]'",
                "Draw specified text at position (x,y) and with specified font size.");
    cimg_option("-image","'[ind_image],[ind_mask],x[%],y[%][,z[%]]' or '[ind_image],x[%],y[%][,z[%]]'","");
    cimg_help("                                              "
              "Draw sprite image (with opacity mask) on specified image(s).");
    cimg_option("-plasma","'alpha[,beta]'","Draw plasma on specified image(s).");
    cimg_option("-mandelbrot","'[z0r[,z0i[,z1r[,z1i[,iter[,julia[,c0r[,c0i]]]]]]]]'",
                "Draw Mandelbrot/Julia fractals on specified image(s).");
    cimg_option("-flood","'x[%][,y[%][,z[%][,tolerance[,color]]]]'",
                "Flood-fill image(s) starting at (x,y,z) with specified tolerance and color.");

    cimg_help("\n  Commands : List manipulation\n"
              "  ----------------------------");
    cimg_option("-remove","(no args)","Remove image(s) from list.");
    cimg_option("-keep","(no args)","Keep only specified image(s) (eq. to '-k').");
    cimg_option("-move","['indice']","Move image(s) at specified position.");
    cimg_option("-reverse","(no args)","Reverse image(s) order.");
    cimg_option("-name","\"name\"","Set display name of image(s).");

    cimg_help("\n  Commands : 3D objects\n"
              "  ---------------------");
    cimg_option("-elevation","'[z-factor]'",
                "Build 3D elevation of image(s) using specified Z-factor.");
    cimg_option("-invert3d","(noargs)","Invert orientation of 3D object(s).");
    cimg_option("-isovalue","'value'","Retrieve isophote or isosurface of image(s).");

    cimg_help("\n  Commands : Input/Output\n"
              "  -----------------------");
    cimg_option("-input","'filename' or 'width[%][xheight[%][xdepth[%][xdim[%][xN]]]]' or '[indice][xN]'","");
    cimg_help("                                              "
              "Input filename, empty image or image copy (eq. to '-i' or (no args)).");
    cimg_option("-output","'filename'","Output image(s) in specified file (eq. to '-o').");
    cimg_option("-print","(no args)","Print image(s) informations (eq. to '-p').");
    cimg_option("-display","(no args)","Display image(s) (eq. to '-d').");
    cimg_option("-display3d","(no args)","Display 3D objects (eq. to '-d3d').");
    cimg_option("-plot","'[plot_type[,value_min[,value_max]]]'",
                "Display image(s) as 1D plot(s) in specified value range");
    cimg_help("                                              "
              "('plot_type' can be '{0=dots, 1=lines, 2=bars, 3=splines}').");
    cimg_option("-quit","(no args)","Quit.");

    cimg_help("\n  Commands : Custom macros\n"
              "  ------------------------");
    unsigned int nb_macros = macros.size;
    cimglist_for(macros,l) if (macros(l,0)=='_') --nb_macros;
    if (!nb_macros) std::fprintf(stderr,"  No custom macro commands are defined in G'MIC.");
    else {
      std::fprintf(stderr,"  Currently, there are %u custom macro command%s recognized by G'MIC :\n",nb_macros,nb_macros>1?"s":"");
      unsigned int curr = 0;
      cimglist_for(macros,l) if (macros(l,0)!='_') {
        if (curr!=nb_macros-1)
          std::fprintf(stderr,"%s%s, ",!(curr%8)?"\n    ":"",macros[l].ptr());
        else
          std::fprintf(stderr,"%s%s.",!(curr%8)?"\n    ":"",macros[l].ptr());
        ++curr;
      }
    }

    cimg_help("\n\n  Viewers shortcuts\n"
              "  -----------------");
    cimg_help("  When displaying images or 3D objects, you may use these shortcuts in G'MIC viewers :");
    cimg_help("   - CTRL+D : Increase window size.");
    cimg_help("   - CTRL+C : Decrease window size.");
    cimg_help("   - CTRL+R : Reset window size.");
    cimg_help("   - CTRL+F : Toggle fullscreen.");
    cimg_help("   - CTRL+S : Save window snapshot.");
    cimg_help("   - CTRL+O : Save current image or 3D object.\n");
    cimg_help("  Specific options for image viewer :");
    cimg_help("   - CTRL+P             : Play stack of frames as a movie.");
    cimg_help("   - CTRL+(mousewheel)  : Zoom in/out.");
    cimg_help("   - SHIFT+(mousewheel) : Go left/right.");
    cimg_help("   - ALT+(mousewheel)   : Go up/down.");
    cimg_help("   - BACKSPACE          : Reset zoom.");
    cimg_help("  Specific options for 3D object viewer :");
    cimg_help("   - CTRL + Z : Enable/disable Z-buffer rendering");

    cimg_help("\n  Filename options\n"
              "  ----------------");
    cimg_help("  G'MIC is able to read/write most of the classical file formats, including :");
    cimg_help("   - 2D grayscale/color images : PNG,JPEG,GIF,PNM,TIFF,BMP,....");
    cimg_help("   - 3D volumetric images : DICOM,HDR,NII,PAN,CIMG,INR,....");
    cimg_help("   - Video files : MPEG,AVI,MOV,OGG,FLV,...");
    cimg_help("   - Data files : DLM,ASC,RAW,TXT.\n");
    cimg_help("  Specific options :");
    cimg_help("   - For video files : you may write 'video.ext:first_frame,last_frame,step'");
    cimg_help("     to read only sub-frames of the image sequence.");
    cimg_help("   - For RAW files : you can specify the image dimensions like this :");
    cimg_help("     'file.raw:widthxheightxdepthxdim'.");
    cimg_help("   - For YUV files : you must specify image dimensions and may read only");
    cimg_help("     sub-frames of the image sequence, by specifying :");
    cimg_help("     'file.yuv:widthxheight,first_frame,last_frame,step'.");

    cimg_help("\n  Examples of use\n"
              "  ---------------");
    cimg_help("   - View image data : ");
    cimg_help("     gmic file1.bmp file2.jpeg");
    cimg_help("   - Convert image files : ");
    cimg_help("     gmic input.bmp -o output.jpg");
    cimg_help("   - Create volumetric image(s) from movie sequence : ");
    cimg_help("     gmic input.mpg -a z -o output.hdr");
    cimg_help("   - Compute image gradient norm : ");
    cimg_help("     gmic input.bmp -gradient -a v -norm");
    cimg_help("   - Compute image laplacian, using convolution : ");
    cimg_help("     gmic input.bmp 3x3 -fill[1] 0,1,0,1,-4,1,0,1,0 -convolve[0] [1] -keep[0]");
    cimg_help("   - Create G'MIC 3D logo : ");
    cimg_help("     gmic 180x70x1x3 -text G\\'MIC,30,5,50,1 -blur 2 -n 0,100 [0] -plasma[1] \\");
    cimg_help("     -+ -blur 1 -elevation -0.1 -rd3d 4");
    cimg_help("\n  See also the macros defined in the provided macro file 'gmic_def.raw' for other examples.");

    cimg_help("\n  ** G'MIC comes with ABSOLUTELY NO WARRANTY; "
              "for details visit http://gmic.sourceforge.net **");
    std::exit(0);
  }

  // Construct new command line by substituting macros.
  const CImg<char> zero(1,1,1,1,0);
  for (int i = 0; i<argc; ++i) command_line.insert(CImg<char>(argv[i],cimg::strlen(argv[i])+1,1,1,1,false));
  cimglist_for(command_line,i) {
    const CImg<char> &item = command_line[i];
    cimglist_for(macros,l) if (item[0]=='-' && !cimg::strcmp(item.ptr()+1,macros[l].ptr())) {
      CImgList<char> command = commands[l].get_split(' ',false,false);
      cimglist_for(command,k) command[k].append(zero,'y');
      command_line.remove(i); command_line.insert(command,i);
      --i; break;
    }
    if (command_line.size>65535) gmic_error("Memory overflow in macro substitution, check for recursive macros !");
  }
  gmic_globals::argc = command_line.size;
  gmic_globals::argv = new char*[command_line.size];
  cimglist_for(command_line,l) gmic_globals::argv[l] = command_line[l].ptr();

  // Call typed parsing procedure.
  gmic_print("Begin.");
  CImgList<float> images;
  gmic_parse_float(images);
  gmic_print("Quit.\n");
  delete[] gmic_globals::argv;
  std::exit(0);
  return 0;
}
#endif
